/* Generated by Cython 0.28.5 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "include_dirs": [
            "brainpy/_c/"
        ],
        "name": "brainpy._c.isotopic_constants",
        "sources": [
            "brainpy/_c/isotopic_constants.pyx"
        ]
    },
    "module_name": "brainpy._c.isotopic_constants"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_28_5"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0; // PyThread_create_key reports success always
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif // TSS (Thread Specific Storage) API
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
  #define PyObject_Unicode             PyObject_Str
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__brainpy___c__isotopic_constants
#define __PYX_HAVE_API__brainpy___c__isotopic_constants
/* Early includes */
#include <string.h>
#include <stdlib.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "brainpy\\_c\\isotopic_constants.pyx",
  "brainpy\\_c\\composition.pxd",
};

/* "brainpy/_c/composition.pxd":3
 * from brainpy.mass_dict import nist_mass as __nist_mass
 * 
 * ctypedef int count_type             # <<<<<<<<<<<<<<
 * 
 * cdef dict nist_mass
 */
typedef int __pyx_t_7brainpy_2_c_11composition_count_type;

/*--- Type declarations ---*/
struct __pyx_obj_7brainpy_2_c_11composition_PyComposition;
struct __pyx_opt_args_7brainpy_2_c_11composition_neutral_mass;
struct __pyx_opt_args_7brainpy_2_c_11composition_mass_charge_ratio;
struct __pyx_t_7brainpy_2_c_11composition_Isotope;
struct __pyx_t_7brainpy_2_c_11composition_IsotopeMap;
struct __pyx_t_7brainpy_2_c_11composition_Element;
struct __pyx_t_7brainpy_2_c_11composition_ElementHashBucket;
struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable;
struct __pyx_t_7brainpy_2_c_11composition_Composition;
struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition__set_impl;

/* "brainpy/_c/composition.pxd":10
 * cdef double PROTON = nist_mass["H+"][0][0]
 * 
 * cdef double neutral_mass(double mz,  int z, double charge_carrier=*) nogil             # <<<<<<<<<<<<<<
 * cdef double mass_charge_ratio(double neutral_mass, int z, double charge_carrier=*) nogil
 * cdef char* _parse_isotope_string(char* label, int* isotope_num, char* element_name) nogil
 */
struct __pyx_opt_args_7brainpy_2_c_11composition_neutral_mass {
  int __pyx_n;
  double charge_carrier;
};

/* "brainpy/_c/composition.pxd":11
 * 
 * cdef double neutral_mass(double mz,  int z, double charge_carrier=*) nogil
 * cdef double mass_charge_ratio(double neutral_mass, int z, double charge_carrier=*) nogil             # <<<<<<<<<<<<<<
 * cdef char* _parse_isotope_string(char* label, int* isotope_num, char* element_name) nogil
 * 
 */
struct __pyx_opt_args_7brainpy_2_c_11composition_mass_charge_ratio {
  int __pyx_n;
  double charge_carrier;
};

/* "brainpy/_c/composition.pxd":17
 * # Isotope and IsotopeMap Declarations
 * 
 * cdef struct Isotope:             # <<<<<<<<<<<<<<
 *     double mass
 *     double abundance
 */
struct __pyx_t_7brainpy_2_c_11composition_Isotope {
  double mass;
  double abundance;
  int neutrons;
  int neutron_shift;
};

/* "brainpy/_c/composition.pxd":23
 *     int neutron_shift
 * 
 * cdef struct IsotopeMap:             # <<<<<<<<<<<<<<
 *     Isotope* bins
 *     size_t size
 */
struct __pyx_t_7brainpy_2_c_11composition_IsotopeMap {
  struct __pyx_t_7brainpy_2_c_11composition_Isotope *bins;
  size_t size;
};

/* "brainpy/_c/composition.pxd":35
 * # Element Declarations
 * 
 * cdef struct Element:             # <<<<<<<<<<<<<<
 *     char* symbol
 *     IsotopeMap* isotopes
 */
struct __pyx_t_7brainpy_2_c_11composition_Element {
  char *symbol;
  struct __pyx_t_7brainpy_2_c_11composition_IsotopeMap *isotopes;
  int monoisotopic_isotope_index;
};

/* "brainpy/_c/composition.pxd":54
 * # ElementHashTable and ElementHashBucket Declarations
 * 
 * cdef struct ElementHashBucket:             # <<<<<<<<<<<<<<
 *     Element** elements
 *     size_t used
 */
struct __pyx_t_7brainpy_2_c_11composition_ElementHashBucket {
  struct __pyx_t_7brainpy_2_c_11composition_Element **elements;
  size_t used;
  size_t size;
};

/* "brainpy/_c/composition.pxd":63
 * 
 * 
 * cdef struct ElementHashTable:             # <<<<<<<<<<<<<<
 *     ElementHashBucket* buckets
 *     size_t size
 */
struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable {
  struct __pyx_t_7brainpy_2_c_11composition_ElementHashBucket *buckets;
  size_t size;
};

/* "brainpy/_c/composition.pxd":92
 * # Composition Declarations
 * 
 * cdef struct Composition:             # <<<<<<<<<<<<<<
 *     char** elements
 *     count_type* counts
 */
struct __pyx_t_7brainpy_2_c_11composition_Composition {
  char **elements;
  __pyx_t_7brainpy_2_c_11composition_count_type *counts;
  size_t size;
  size_t used;
  int max_variants;
};

/* "brainpy/_c/composition.pxd":124
 *     @staticmethod
 *     cdef PyComposition _create(Composition* base)
 *     cdef void _set_impl(self, Composition* composition, bint free_existing=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef double mass(self)
 */
struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition__set_impl {
  int __pyx_n;
  int free_existing;
};
struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector;

/* "brainpy/_c/double_vector.pxd":1
 * cdef struct DoubleVector:             # <<<<<<<<<<<<<<
 *     double* v
 *     size_t used
 */
struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector {
  double *v;
  size_t used;
  size_t size;
};
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters;
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants;
struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants;

/* "brainpy/_c/isotopic_constants.pxd":6
 * from brainpy._c.double_vector cimport DoubleVector
 * 
 * ctypedef DoubleVector dvec             # <<<<<<<<<<<<<<
 * 
 * 
 */
typedef struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector __pyx_t_7brainpy_2_c_18isotopic_constants_dvec;

/* "brainpy/_c/isotopic_constants.pxd":9
 * 
 * 
 * cdef struct PolynomialParameters:             # <<<<<<<<<<<<<<
 *     dvec* elementary_symmetric_polynomial
 *     dvec* power_sum
 */
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters {
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *elementary_symmetric_polynomial;
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *power_sum;
};

/* "brainpy/_c/isotopic_constants.pxd":14
 * 
 * 
 * cdef struct PhiConstants:             # <<<<<<<<<<<<<<
 *     int order
 *     Element* element
 */
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants {
  int order;
  struct __pyx_t_7brainpy_2_c_11composition_Element *element;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *element_coefficients;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *mass_coefficients;
};

/* "brainpy/_c/isotopic_constants.pxd":21
 * 
 * 
 * cdef struct IsotopicConstants:             # <<<<<<<<<<<<<<
 *     int order
 *     PhiConstants** constants
 */
struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants {
  int order;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants **constants;
  size_t size;
  size_t used;
};

/* "brainpy/_c/composition.pxd":117
 * cdef Composition* dict_to_composition(dict comp_dict)
 * 
 * cdef class PyComposition(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Composition* impl
 */
struct __pyx_obj_7brainpy_2_c_11composition_PyComposition {
  PyObject_HEAD
  struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition *__pyx_vtab;
  struct __pyx_t_7brainpy_2_c_11composition_Composition *impl;
  double cached_mass;
  int _clean;
};



struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition {
  struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *(*_create)(struct __pyx_t_7brainpy_2_c_11composition_Composition *);
  void (*_set_impl)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, struct __pyx_t_7brainpy_2_c_11composition_Composition *, struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition__set_impl *__pyx_optional_args);
  double (*mass)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);
  int (*__pyx___equality_pycomposition)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);
  int (*__pyx___equality_dict)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *(*copy)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition *__pyx_vtabptr_7brainpy_2_c_11composition_PyComposition;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* Print.proto */
static int __Pyx_Print(PyObject*, PyObject *, int);
#if CYTHON_COMPILING_IN_PYPY || PY_MAJOR_VERSION >= 3
static PyObject* __pyx_print = 0;
static PyObject* __pyx_print_kwargs = 0;
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* PrintOne.proto */
static int __Pyx_PrintOne(PyObject* stream, PyObject *o);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* VoidPtrExport.proto */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'brainpy._c.composition' */
static PyTypeObject *__pyx_ptype_7brainpy_2_c_11composition_PyComposition = 0;
static PyObject **__pyx_vp_7brainpy_2_c_11composition_nist_mass = 0;
#define __pyx_v_7brainpy_2_c_11composition_nist_mass (*__pyx_vp_7brainpy_2_c_11composition_nist_mass)
static double *__pyx_vp_7brainpy_2_c_11composition_PROTON = 0;
#define __pyx_v_7brainpy_2_c_11composition_PROTON (*__pyx_vp_7brainpy_2_c_11composition_PROTON)
static struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable **__pyx_vp_7brainpy_2_c_11composition__ElementTable = 0;
#define __pyx_v_7brainpy_2_c_11composition__ElementTable (*__pyx_vp_7brainpy_2_c_11composition__ElementTable)
static char *(*__pyx_f_7brainpy_2_c_11composition__parse_isotope_string)(char *, int *, char *); /*proto*/
static int (*__pyx_f_7brainpy_2_c_11composition_element_max_neutron_shift)(struct __pyx_t_7brainpy_2_c_11composition_Element *); /*proto*/
static struct __pyx_t_7brainpy_2_c_11composition_Element *(*__pyx_f_7brainpy_2_c_11composition_make_fixed_isotope_element)(struct __pyx_t_7brainpy_2_c_11composition_Element *, int); /*proto*/
static struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *(*__pyx_f_7brainpy_2_c_11composition_make_element_hash_table)(size_t); /*proto*/
static int (*__pyx_f_7brainpy_2_c_11composition_element_hash_table_get)(struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *, char *, struct __pyx_t_7brainpy_2_c_11composition_Element **); /*proto*/
static int (*__pyx_f_7brainpy_2_c_11composition_element_hash_table_put)(struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *, struct __pyx_t_7brainpy_2_c_11composition_Element *); /*proto*/
static size_t (*__pyx_f_7brainpy_2_c_11composition_free_element_hash_table)(struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *); /*proto*/

/* Module declarations from 'brainpy._c.double_vector' */
static struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *(*__pyx_f_7brainpy_2_c_13double_vector_make_double_vector_with_size)(size_t); /*proto*/
static struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *(*__pyx_f_7brainpy_2_c_13double_vector_make_double_vector)(void); /*proto*/
static int (*__pyx_f_7brainpy_2_c_13double_vector_double_vector_append)(struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *, double); /*proto*/
static void (*__pyx_f_7brainpy_2_c_13double_vector_free_double_vector)(struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *); /*proto*/
static void (*__pyx_f_7brainpy_2_c_13double_vector_print_double_vector)(struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *); /*proto*/
static void (*__pyx_f_7brainpy_2_c_13double_vector_reset_double_vector)(struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *); /*proto*/
static PyObject *(*__pyx_f_7brainpy_2_c_13double_vector_double_vector_to_list)(struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *); /*proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'libc' */

/* Module declarations from 'brainpy._c.isotopic_constants' */
static size_t __pyx_v_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE;
static __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_f_7brainpy_2_c_18isotopic_constants_vietes(__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *); /*proto*/
static void __pyx_f_7brainpy_2_c_18isotopic_constants__update_power_sum(__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, int); /*proto*/
static void __pyx_f_7brainpy_2_c_18isotopic_constants__update_elementary_symmetric_polynomial(__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, int); /*proto*/
static void __pyx_f_7brainpy_2_c_18isotopic_constants_newton(__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, int); /*proto*/
static __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_f_7brainpy_2_c_18isotopic_constants_compute_isotopic_coefficients(struct __pyx_t_7brainpy_2_c_11composition_Element *, int, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *); /*proto*/
static struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *__pyx_f_7brainpy_2_c_18isotopic_constants_make_polynomial_parameters(struct __pyx_t_7brainpy_2_c_11composition_Element *, int, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *); /*proto*/
static void __pyx_f_7brainpy_2_c_18isotopic_constants_print_polynomial_parameters(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *); /*proto*/
static void __pyx_f_7brainpy_2_c_18isotopic_constants_free_polynomial_parameters(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *); /*proto*/
static void __pyx_f_7brainpy_2_c_18isotopic_constants_print_phi_constants(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *); /*proto*/
static void __pyx_f_7brainpy_2_c_18isotopic_constants_free_phi_constants(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *); /*proto*/
static struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_f_7brainpy_2_c_18isotopic_constants_make_isotopic_constants(void); /*proto*/
static int __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_resize(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *); /*proto*/
static void __pyx_f_7brainpy_2_c_18isotopic_constants_free_isotopic_constants(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *); /*proto*/
static void __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_add_element(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *, char *); /*proto*/
static int __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *, char *, struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants **); /*proto*/
static int __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get_by_index(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *, size_t, struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants **); /*proto*/
#define __Pyx_MODULE_NAME "brainpy._c.isotopic_constants"
extern int __pyx_module_is_main_brainpy___c__isotopic_constants;
int __pyx_module_is_main_brainpy___c__isotopic_constants = 0;

/* Implementation of 'brainpy._c.isotopic_constants' */
static PyObject *__pyx_builtin_range;
static const char __pyx_k_ic[] = "ic";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_sym[] = "sym";
static const char __pyx_k_Nope[] = "Nope";
static const char __pyx_k_elem[] = "elem";
static const char __pyx_k_file[] = "file";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_main_2[] = "main";
static const char __pyx_k_constant[] = "constant";
static const char __pyx_k_pyx_capi[] = "__pyx_capi__";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_coefficients[] = "coefficients";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_brainpy__c_isotopic_constants[] = "brainpy._c.isotopic_constants";
static const char __pyx_k_DEFAULT_ISOTOPIC_CONSTANTS_SIZE[] = "DEFAULT_ISOTOPIC_CONSTANTS_SIZE";
static const char __pyx_k_elementary_symmetric_polynomial[] = "elementary_symmetric_polynomial";
static const char __pyx_k_brainpy__c_isotopic_constants_py[] = "brainpy\\_c\\isotopic_constants.pyx";
static PyObject *__pyx_n_s_DEFAULT_ISOTOPIC_CONSTANTS_SIZE;
static PyObject *__pyx_n_s_Nope;
static PyObject *__pyx_n_s_brainpy__c_isotopic_constants;
static PyObject *__pyx_kp_s_brainpy__c_isotopic_constants_py;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_coefficients;
static PyObject *__pyx_n_s_constant;
static PyObject *__pyx_n_s_elem;
static PyObject *__pyx_n_s_elementary_symmetric_polynomial;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_n_s_file;
static PyObject *__pyx_n_s_ic;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_main_2;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_pyx_capi;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_sym;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_pf_7brainpy_2_c_18isotopic_constants_main(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_tuple_;
static PyObject *__pyx_codeobj__2;
/* Late includes */

/* "brainpy/_c/isotopic_constants.pyx":27
 * 
 * @cython.cdivision
 * cdef dvec* vietes(dvec* coefficients) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         DoubleVector* elementary_symmetric_polynomial
 */

static __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_f_7brainpy_2_c_18isotopic_constants_vietes(__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_coefficients) {
  struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *__pyx_v_elementary_symmetric_polynomial;
  size_t __pyx_v_i;
  double __pyx_v_tail;
  int __pyx_v_sign;
  double __pyx_v_el;
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_r;
  size_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;

  /* "brainpy/_c/isotopic_constants.pyx":35
 *         double el
 * 
 *     elementary_symmetric_polynomial = make_double_vector_with_size(             # <<<<<<<<<<<<<<
 *         coefficients.used)
 *     tail = coefficients.v[coefficients.used - 1]
 */
  __pyx_v_elementary_symmetric_polynomial = __pyx_f_7brainpy_2_c_13double_vector_make_double_vector_with_size(__pyx_v_coefficients->used);

  /* "brainpy/_c/isotopic_constants.pyx":37
 *     elementary_symmetric_polynomial = make_double_vector_with_size(
 *         coefficients.used)
 *     tail = coefficients.v[coefficients.used - 1]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(coefficients.used):
 */
  __pyx_v_tail = (__pyx_v_coefficients->v[(__pyx_v_coefficients->used - 1)]);

  /* "brainpy/_c/isotopic_constants.pyx":39
 *     tail = coefficients.v[coefficients.used - 1]
 * 
 *     for i in range(coefficients.used):             # <<<<<<<<<<<<<<
 *         sign = 1 if (i % 2) == 0 else -1
 *         el = sign * coefficients.v[coefficients.used - i - 1] / tail
 */
  __pyx_t_1 = __pyx_v_coefficients->used;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "brainpy/_c/isotopic_constants.pyx":40
 * 
 *     for i in range(coefficients.used):
 *         sign = 1 if (i % 2) == 0 else -1             # <<<<<<<<<<<<<<
 *         el = sign * coefficients.v[coefficients.used - i - 1] / tail
 *         double_vector_append(elementary_symmetric_polynomial, el)
 */
    if ((((__pyx_v_i % 2) == 0) != 0)) {
      __pyx_t_4 = 1;
    } else {
      __pyx_t_4 = -1;
    }
    __pyx_v_sign = __pyx_t_4;

    /* "brainpy/_c/isotopic_constants.pyx":41
 *     for i in range(coefficients.used):
 *         sign = 1 if (i % 2) == 0 else -1
 *         el = sign * coefficients.v[coefficients.used - i - 1] / tail             # <<<<<<<<<<<<<<
 *         double_vector_append(elementary_symmetric_polynomial, el)
 * 
 */
    __pyx_v_el = ((__pyx_v_sign * (__pyx_v_coefficients->v[((__pyx_v_coefficients->used - __pyx_v_i) - 1)])) / __pyx_v_tail);

    /* "brainpy/_c/isotopic_constants.pyx":42
 *         sign = 1 if (i % 2) == 0 else -1
 *         el = sign * coefficients.v[coefficients.used - i - 1] / tail
 *         double_vector_append(elementary_symmetric_polynomial, el)             # <<<<<<<<<<<<<<
 * 
 *     return elementary_symmetric_polynomial
 */
    (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_elementary_symmetric_polynomial, __pyx_v_el));
  }

  /* "brainpy/_c/isotopic_constants.pyx":44
 *         double_vector_append(elementary_symmetric_polynomial, el)
 * 
 *     return elementary_symmetric_polynomial             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_elementary_symmetric_polynomial;
  goto __pyx_L0;

  /* "brainpy/_c/isotopic_constants.pyx":27
 * 
 * @cython.cdivision
 * cdef dvec* vietes(dvec* coefficients) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         DoubleVector* elementary_symmetric_polynomial
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":47
 * 
 * 
 * cdef void _update_power_sum(dvec* ps_vec, dvec* esp_vec, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t begin, end, k, j
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants__update_power_sum(__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_ps_vec, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_esp_vec, CYTHON_UNUSED int __pyx_v_order) {
  size_t __pyx_v_begin;
  size_t __pyx_v_end;
  size_t __pyx_v_k;
  size_t __pyx_v_j;
  int __pyx_v_sign;
  double __pyx_v_temp_ps;
  size_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;

  /* "brainpy/_c/isotopic_constants.pyx":53
 *         double temp_ps
 * 
 *     begin = ps_vec.used             # <<<<<<<<<<<<<<
 *     end = esp_vec.used
 * 
 */
  __pyx_t_1 = __pyx_v_ps_vec->used;
  __pyx_v_begin = __pyx_t_1;

  /* "brainpy/_c/isotopic_constants.pyx":54
 * 
 *     begin = ps_vec.used
 *     end = esp_vec.used             # <<<<<<<<<<<<<<
 * 
 *     for k in range(begin, end):
 */
  __pyx_t_1 = __pyx_v_esp_vec->used;
  __pyx_v_end = __pyx_t_1;

  /* "brainpy/_c/isotopic_constants.pyx":56
 *     end = esp_vec.used
 * 
 *     for k in range(begin, end):             # <<<<<<<<<<<<<<
 *         if k == 0:
 *             double_vector_append(ps_vec, 0.0)
 */
  __pyx_t_1 = __pyx_v_end;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = __pyx_v_begin; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_k = __pyx_t_3;

    /* "brainpy/_c/isotopic_constants.pyx":57
 * 
 *     for k in range(begin, end):
 *         if k == 0:             # <<<<<<<<<<<<<<
 *             double_vector_append(ps_vec, 0.0)
 *             continue
 */
    __pyx_t_4 = ((__pyx_v_k == 0) != 0);
    if (__pyx_t_4) {

      /* "brainpy/_c/isotopic_constants.pyx":58
 *     for k in range(begin, end):
 *         if k == 0:
 *             double_vector_append(ps_vec, 0.0)             # <<<<<<<<<<<<<<
 *             continue
 *         temp_ps = 0.
 */
      (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_ps_vec, 0.0));

      /* "brainpy/_c/isotopic_constants.pyx":59
 *         if k == 0:
 *             double_vector_append(ps_vec, 0.0)
 *             continue             # <<<<<<<<<<<<<<
 *         temp_ps = 0.
 *         sign = -1
 */
      goto __pyx_L3_continue;

      /* "brainpy/_c/isotopic_constants.pyx":57
 * 
 *     for k in range(begin, end):
 *         if k == 0:             # <<<<<<<<<<<<<<
 *             double_vector_append(ps_vec, 0.0)
 *             continue
 */
    }

    /* "brainpy/_c/isotopic_constants.pyx":60
 *             double_vector_append(ps_vec, 0.0)
 *             continue
 *         temp_ps = 0.             # <<<<<<<<<<<<<<
 *         sign = -1
 * 
 */
    __pyx_v_temp_ps = 0.;

    /* "brainpy/_c/isotopic_constants.pyx":61
 *             continue
 *         temp_ps = 0.
 *         sign = -1             # <<<<<<<<<<<<<<
 * 
 *         for j in range(1, k):
 */
    __pyx_v_sign = -1;

    /* "brainpy/_c/isotopic_constants.pyx":63
 *         sign = -1
 * 
 *         for j in range(1, k):             # <<<<<<<<<<<<<<
 *             sign *= -1
 *             temp_ps += sign * (esp_vec.v[j]) * (ps_vec.v[k - j])
 */
    __pyx_t_5 = __pyx_v_k;
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = 1; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_j = __pyx_t_7;

      /* "brainpy/_c/isotopic_constants.pyx":64
 * 
 *         for j in range(1, k):
 *             sign *= -1             # <<<<<<<<<<<<<<
 *             temp_ps += sign * (esp_vec.v[j]) * (ps_vec.v[k - j])
 *         sign *= -1
 */
      __pyx_v_sign = (__pyx_v_sign * -1L);

      /* "brainpy/_c/isotopic_constants.pyx":65
 *         for j in range(1, k):
 *             sign *= -1
 *             temp_ps += sign * (esp_vec.v[j]) * (ps_vec.v[k - j])             # <<<<<<<<<<<<<<
 *         sign *= -1
 *         temp_ps += sign * (esp_vec.v[k]) * k
 */
      __pyx_v_temp_ps = (__pyx_v_temp_ps + ((__pyx_v_sign * (__pyx_v_esp_vec->v[__pyx_v_j])) * (__pyx_v_ps_vec->v[(__pyx_v_k - __pyx_v_j)])));
    }

    /* "brainpy/_c/isotopic_constants.pyx":66
 *             sign *= -1
 *             temp_ps += sign * (esp_vec.v[j]) * (ps_vec.v[k - j])
 *         sign *= -1             # <<<<<<<<<<<<<<
 *         temp_ps += sign * (esp_vec.v[k]) * k
 *         double_vector_append(ps_vec, temp_ps)
 */
    __pyx_v_sign = (__pyx_v_sign * -1L);

    /* "brainpy/_c/isotopic_constants.pyx":67
 *             temp_ps += sign * (esp_vec.v[j]) * (ps_vec.v[k - j])
 *         sign *= -1
 *         temp_ps += sign * (esp_vec.v[k]) * k             # <<<<<<<<<<<<<<
 *         double_vector_append(ps_vec, temp_ps)
 * 
 */
    __pyx_v_temp_ps = (__pyx_v_temp_ps + ((__pyx_v_sign * (__pyx_v_esp_vec->v[__pyx_v_k])) * __pyx_v_k));

    /* "brainpy/_c/isotopic_constants.pyx":68
 *         sign *= -1
 *         temp_ps += sign * (esp_vec.v[k]) * k
 *         double_vector_append(ps_vec, temp_ps)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_ps_vec, __pyx_v_temp_ps));
    __pyx_L3_continue:;
  }

  /* "brainpy/_c/isotopic_constants.pyx":47
 * 
 * 
 * cdef void _update_power_sum(dvec* ps_vec, dvec* esp_vec, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t begin, end, k, j
 */

  /* function exit code */
}

/* "brainpy/_c/isotopic_constants.pyx":72
 * 
 * @cython.cdivision
 * cdef void _update_elementary_symmetric_polynomial(dvec* ps_vec, dvec* esp_vec, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t begin, end, k, j
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants__update_elementary_symmetric_polynomial(__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_ps_vec, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_esp_vec, int __pyx_v_order) {
  size_t __pyx_v_begin;
  size_t __pyx_v_end;
  size_t __pyx_v_k;
  size_t __pyx_v_j;
  int __pyx_v_sign;
  double __pyx_v_el;
  size_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  int __pyx_t_8;

  /* "brainpy/_c/isotopic_constants.pyx":78
 *         double el
 * 
 *     begin = esp_vec.used             # <<<<<<<<<<<<<<
 *     end = ps_vec.used
 * 
 */
  __pyx_t_1 = __pyx_v_esp_vec->used;
  __pyx_v_begin = __pyx_t_1;

  /* "brainpy/_c/isotopic_constants.pyx":79
 * 
 *     begin = esp_vec.used
 *     end = ps_vec.used             # <<<<<<<<<<<<<<
 * 
 *     for k in range(begin, end):
 */
  __pyx_t_1 = __pyx_v_ps_vec->used;
  __pyx_v_end = __pyx_t_1;

  /* "brainpy/_c/isotopic_constants.pyx":81
 *     end = ps_vec.used
 * 
 *     for k in range(begin, end):             # <<<<<<<<<<<<<<
 *         if k == 0:
 *             double_vector_append(esp_vec, 1.0)
 */
  __pyx_t_1 = __pyx_v_end;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = __pyx_v_begin; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_k = __pyx_t_3;

    /* "brainpy/_c/isotopic_constants.pyx":82
 * 
 *     for k in range(begin, end):
 *         if k == 0:             # <<<<<<<<<<<<<<
 *             double_vector_append(esp_vec, 1.0)
 *         elif k > order:
 */
    __pyx_t_4 = ((__pyx_v_k == 0) != 0);
    if (__pyx_t_4) {

      /* "brainpy/_c/isotopic_constants.pyx":83
 *     for k in range(begin, end):
 *         if k == 0:
 *             double_vector_append(esp_vec, 1.0)             # <<<<<<<<<<<<<<
 *         elif k > order:
 *             double_vector_append(esp_vec, 0.0)
 */
      (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_esp_vec, 1.0));

      /* "brainpy/_c/isotopic_constants.pyx":82
 * 
 *     for k in range(begin, end):
 *         if k == 0:             # <<<<<<<<<<<<<<
 *             double_vector_append(esp_vec, 1.0)
 *         elif k > order:
 */
      goto __pyx_L5;
    }

    /* "brainpy/_c/isotopic_constants.pyx":84
 *         if k == 0:
 *             double_vector_append(esp_vec, 1.0)
 *         elif k > order:             # <<<<<<<<<<<<<<
 *             double_vector_append(esp_vec, 0.0)
 *         else:
 */
    __pyx_t_4 = ((__pyx_v_k > __pyx_v_order) != 0);
    if (__pyx_t_4) {

      /* "brainpy/_c/isotopic_constants.pyx":85
 *             double_vector_append(esp_vec, 1.0)
 *         elif k > order:
 *             double_vector_append(esp_vec, 0.0)             # <<<<<<<<<<<<<<
 *         else:
 *             el = 0.
 */
      (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_esp_vec, 0.0));

      /* "brainpy/_c/isotopic_constants.pyx":84
 *         if k == 0:
 *             double_vector_append(esp_vec, 1.0)
 *         elif k > order:             # <<<<<<<<<<<<<<
 *             double_vector_append(esp_vec, 0.0)
 *         else:
 */
      goto __pyx_L5;
    }

    /* "brainpy/_c/isotopic_constants.pyx":87
 *             double_vector_append(esp_vec, 0.0)
 *         else:
 *             el = 0.             # <<<<<<<<<<<<<<
 *             for j in range(1, k + 1):
 *                 sign = 1 if (j % 2) == 1 else -1
 */
    /*else*/ {
      __pyx_v_el = 0.;

      /* "brainpy/_c/isotopic_constants.pyx":88
 *         else:
 *             el = 0.
 *             for j in range(1, k + 1):             # <<<<<<<<<<<<<<
 *                 sign = 1 if (j % 2) == 1 else -1
 *                 el += sign * ps_vec.v[j] * esp_vec.v[k - j]
 */
      __pyx_t_5 = (__pyx_v_k + 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = 1; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
        __pyx_v_j = __pyx_t_7;

        /* "brainpy/_c/isotopic_constants.pyx":89
 *             el = 0.
 *             for j in range(1, k + 1):
 *                 sign = 1 if (j % 2) == 1 else -1             # <<<<<<<<<<<<<<
 *                 el += sign * ps_vec.v[j] * esp_vec.v[k - j]
 *             el /= <double>k
 */
        if ((((__pyx_v_j % 2) == 1) != 0)) {
          __pyx_t_8 = 1;
        } else {
          __pyx_t_8 = -1;
        }
        __pyx_v_sign = __pyx_t_8;

        /* "brainpy/_c/isotopic_constants.pyx":90
 *             for j in range(1, k + 1):
 *                 sign = 1 if (j % 2) == 1 else -1
 *                 el += sign * ps_vec.v[j] * esp_vec.v[k - j]             # <<<<<<<<<<<<<<
 *             el /= <double>k
 *             double_vector_append(esp_vec, el)
 */
        __pyx_v_el = (__pyx_v_el + ((__pyx_v_sign * (__pyx_v_ps_vec->v[__pyx_v_j])) * (__pyx_v_esp_vec->v[(__pyx_v_k - __pyx_v_j)])));
      }

      /* "brainpy/_c/isotopic_constants.pyx":91
 *                 sign = 1 if (j % 2) == 1 else -1
 *                 el += sign * ps_vec.v[j] * esp_vec.v[k - j]
 *             el /= <double>k             # <<<<<<<<<<<<<<
 *             double_vector_append(esp_vec, el)
 * 
 */
      __pyx_v_el = (__pyx_v_el / ((double)__pyx_v_k));

      /* "brainpy/_c/isotopic_constants.pyx":92
 *                 el += sign * ps_vec.v[j] * esp_vec.v[k - j]
 *             el /= <double>k
 *             double_vector_append(esp_vec, el)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_esp_vec, __pyx_v_el));
    }
    __pyx_L5:;
  }

  /* "brainpy/_c/isotopic_constants.pyx":72
 * 
 * @cython.cdivision
 * cdef void _update_elementary_symmetric_polynomial(dvec* ps_vec, dvec* esp_vec, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t begin, end, k, j
 */

  /* function exit code */
}

/* "brainpy/_c/isotopic_constants.pyx":95
 * 
 * 
 * cdef void newton(dvec* ps_vec, dvec* esp_vec, int order) nogil:             # <<<<<<<<<<<<<<
 *     if ps_vec.used > esp_vec.used:
 *         _update_elementary_symmetric_polynomial(ps_vec, esp_vec, order)
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants_newton(__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_ps_vec, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_esp_vec, int __pyx_v_order) {
  int __pyx_t_1;

  /* "brainpy/_c/isotopic_constants.pyx":96
 * 
 * cdef void newton(dvec* ps_vec, dvec* esp_vec, int order) nogil:
 *     if ps_vec.used > esp_vec.used:             # <<<<<<<<<<<<<<
 *         _update_elementary_symmetric_polynomial(ps_vec, esp_vec, order)
 *     elif ps_vec.used < esp_vec.used:
 */
  __pyx_t_1 = ((__pyx_v_ps_vec->used > __pyx_v_esp_vec->used) != 0);
  if (__pyx_t_1) {

    /* "brainpy/_c/isotopic_constants.pyx":97
 * cdef void newton(dvec* ps_vec, dvec* esp_vec, int order) nogil:
 *     if ps_vec.used > esp_vec.used:
 *         _update_elementary_symmetric_polynomial(ps_vec, esp_vec, order)             # <<<<<<<<<<<<<<
 *     elif ps_vec.used < esp_vec.used:
 *         _update_power_sum(ps_vec, esp_vec, order)
 */
    __pyx_f_7brainpy_2_c_18isotopic_constants__update_elementary_symmetric_polynomial(__pyx_v_ps_vec, __pyx_v_esp_vec, __pyx_v_order);

    /* "brainpy/_c/isotopic_constants.pyx":96
 * 
 * cdef void newton(dvec* ps_vec, dvec* esp_vec, int order) nogil:
 *     if ps_vec.used > esp_vec.used:             # <<<<<<<<<<<<<<
 *         _update_elementary_symmetric_polynomial(ps_vec, esp_vec, order)
 *     elif ps_vec.used < esp_vec.used:
 */
    goto __pyx_L3;
  }

  /* "brainpy/_c/isotopic_constants.pyx":98
 *     if ps_vec.used > esp_vec.used:
 *         _update_elementary_symmetric_polynomial(ps_vec, esp_vec, order)
 *     elif ps_vec.used < esp_vec.used:             # <<<<<<<<<<<<<<
 *         _update_power_sum(ps_vec, esp_vec, order)
 * 
 */
  __pyx_t_1 = ((__pyx_v_ps_vec->used < __pyx_v_esp_vec->used) != 0);
  if (__pyx_t_1) {

    /* "brainpy/_c/isotopic_constants.pyx":99
 *         _update_elementary_symmetric_polynomial(ps_vec, esp_vec, order)
 *     elif ps_vec.used < esp_vec.used:
 *         _update_power_sum(ps_vec, esp_vec, order)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_f_7brainpy_2_c_18isotopic_constants__update_power_sum(__pyx_v_ps_vec, __pyx_v_esp_vec, __pyx_v_order);

    /* "brainpy/_c/isotopic_constants.pyx":98
 *     if ps_vec.used > esp_vec.used:
 *         _update_elementary_symmetric_polynomial(ps_vec, esp_vec, order)
 *     elif ps_vec.used < esp_vec.used:             # <<<<<<<<<<<<<<
 *         _update_power_sum(ps_vec, esp_vec, order)
 * 
 */
  }
  __pyx_L3:;

  /* "brainpy/_c/isotopic_constants.pyx":95
 * 
 * 
 * cdef void newton(dvec* ps_vec, dvec* esp_vec, int order) nogil:             # <<<<<<<<<<<<<<
 *     if ps_vec.used > esp_vec.used:
 *         _update_elementary_symmetric_polynomial(ps_vec, esp_vec, order)
 */

  /* function exit code */
}

/* "brainpy/_c/isotopic_constants.pyx":102
 * 
 * 
 * cdef dvec* compute_isotopic_coefficients(Element* element, bint with_mass, dvec* accumulator) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         int max_isotope_number, current_order
 */

static __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_f_7brainpy_2_c_18isotopic_constants_compute_isotopic_coefficients(struct __pyx_t_7brainpy_2_c_11composition_Element *__pyx_v_element, int __pyx_v_with_mass, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_accumulator) {
  int __pyx_v_max_isotope_number;
  int __pyx_v_current_order;
  struct __pyx_t_7brainpy_2_c_11composition_Isotope *__pyx_v_isotope;
  double __pyx_v_coef;
  size_t __pyx_v_i;
  CYTHON_UNUSED size_t __pyx_v_j;
  size_t __pyx_v_k;
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_r;
  size_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  size_t __pyx_t_8;

  /* "brainpy/_c/isotopic_constants.pyx":109
 *         size_t i, j, k
 * 
 *     max_isotope_number = element_max_neutron_shift(element)             # <<<<<<<<<<<<<<
 *     for i in range(element.isotopes.size):
 *         k = element.isotopes.size - i - 1
 */
  __pyx_v_max_isotope_number = __pyx_f_7brainpy_2_c_11composition_element_max_neutron_shift(__pyx_v_element);

  /* "brainpy/_c/isotopic_constants.pyx":110
 * 
 *     max_isotope_number = element_max_neutron_shift(element)
 *     for i in range(element.isotopes.size):             # <<<<<<<<<<<<<<
 *         k = element.isotopes.size - i - 1
 *         isotope = &(element.isotopes.bins[k])
 */
  __pyx_t_1 = __pyx_v_element->isotopes->size;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "brainpy/_c/isotopic_constants.pyx":111
 *     max_isotope_number = element_max_neutron_shift(element)
 *     for i in range(element.isotopes.size):
 *         k = element.isotopes.size - i - 1             # <<<<<<<<<<<<<<
 *         isotope = &(element.isotopes.bins[k])
 *         current_order = max_isotope_number - isotope.neutron_shift
 */
    __pyx_v_k = ((__pyx_v_element->isotopes->size - __pyx_v_i) - 1);

    /* "brainpy/_c/isotopic_constants.pyx":112
 *     for i in range(element.isotopes.size):
 *         k = element.isotopes.size - i - 1
 *         isotope = &(element.isotopes.bins[k])             # <<<<<<<<<<<<<<
 *         current_order = max_isotope_number - isotope.neutron_shift
 *         if with_mass:
 */
    __pyx_v_isotope = (&(__pyx_v_element->isotopes->bins[__pyx_v_k]));

    /* "brainpy/_c/isotopic_constants.pyx":113
 *         k = element.isotopes.size - i - 1
 *         isotope = &(element.isotopes.bins[k])
 *         current_order = max_isotope_number - isotope.neutron_shift             # <<<<<<<<<<<<<<
 *         if with_mass:
 *             coef = isotope.mass
 */
    __pyx_v_current_order = (__pyx_v_max_isotope_number - __pyx_v_isotope->neutron_shift);

    /* "brainpy/_c/isotopic_constants.pyx":114
 *         isotope = &(element.isotopes.bins[k])
 *         current_order = max_isotope_number - isotope.neutron_shift
 *         if with_mass:             # <<<<<<<<<<<<<<
 *             coef = isotope.mass
 *         else:
 */
    __pyx_t_4 = (__pyx_v_with_mass != 0);
    if (__pyx_t_4) {

      /* "brainpy/_c/isotopic_constants.pyx":115
 *         current_order = max_isotope_number - isotope.neutron_shift
 *         if with_mass:
 *             coef = isotope.mass             # <<<<<<<<<<<<<<
 *         else:
 *             coef = 1.0
 */
      __pyx_t_5 = __pyx_v_isotope->mass;
      __pyx_v_coef = __pyx_t_5;

      /* "brainpy/_c/isotopic_constants.pyx":114
 *         isotope = &(element.isotopes.bins[k])
 *         current_order = max_isotope_number - isotope.neutron_shift
 *         if with_mass:             # <<<<<<<<<<<<<<
 *             coef = isotope.mass
 *         else:
 */
      goto __pyx_L5;
    }

    /* "brainpy/_c/isotopic_constants.pyx":117
 *             coef = isotope.mass
 *         else:
 *             coef = 1.0             # <<<<<<<<<<<<<<
 *         if current_order > accumulator.used:
 *             for j in range(accumulator.used, current_order):
 */
    /*else*/ {
      __pyx_v_coef = 1.0;
    }
    __pyx_L5:;

    /* "brainpy/_c/isotopic_constants.pyx":118
 *         else:
 *             coef = 1.0
 *         if current_order > accumulator.used:             # <<<<<<<<<<<<<<
 *             for j in range(accumulator.used, current_order):
 *                 double_vector_append(accumulator, 0.)
 */
    __pyx_t_4 = ((__pyx_v_current_order > __pyx_v_accumulator->used) != 0);
    if (__pyx_t_4) {

      /* "brainpy/_c/isotopic_constants.pyx":119
 *             coef = 1.0
 *         if current_order > accumulator.used:
 *             for j in range(accumulator.used, current_order):             # <<<<<<<<<<<<<<
 *                 double_vector_append(accumulator, 0.)
 *             double_vector_append(accumulator, isotope.abundance * coef)
 */
      __pyx_t_6 = __pyx_v_current_order;
      __pyx_t_7 = __pyx_t_6;
      for (__pyx_t_8 = __pyx_v_accumulator->used; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
        __pyx_v_j = __pyx_t_8;

        /* "brainpy/_c/isotopic_constants.pyx":120
 *         if current_order > accumulator.used:
 *             for j in range(accumulator.used, current_order):
 *                 double_vector_append(accumulator, 0.)             # <<<<<<<<<<<<<<
 *             double_vector_append(accumulator, isotope.abundance * coef)
 *         elif current_order == accumulator.used:
 */
        (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_accumulator, 0.));
      }

      /* "brainpy/_c/isotopic_constants.pyx":121
 *             for j in range(accumulator.used, current_order):
 *                 double_vector_append(accumulator, 0.)
 *             double_vector_append(accumulator, isotope.abundance * coef)             # <<<<<<<<<<<<<<
 *         elif current_order == accumulator.used:
 *             double_vector_append(accumulator, isotope.abundance * coef)
 */
      (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_accumulator, (__pyx_v_isotope->abundance * __pyx_v_coef)));

      /* "brainpy/_c/isotopic_constants.pyx":118
 *         else:
 *             coef = 1.0
 *         if current_order > accumulator.used:             # <<<<<<<<<<<<<<
 *             for j in range(accumulator.used, current_order):
 *                 double_vector_append(accumulator, 0.)
 */
      goto __pyx_L6;
    }

    /* "brainpy/_c/isotopic_constants.pyx":122
 *                 double_vector_append(accumulator, 0.)
 *             double_vector_append(accumulator, isotope.abundance * coef)
 *         elif current_order == accumulator.used:             # <<<<<<<<<<<<<<
 *             double_vector_append(accumulator, isotope.abundance * coef)
 *         else:
 */
    __pyx_t_4 = ((__pyx_v_current_order == __pyx_v_accumulator->used) != 0);
    if (__pyx_t_4) {

      /* "brainpy/_c/isotopic_constants.pyx":123
 *             double_vector_append(accumulator, isotope.abundance * coef)
 *         elif current_order == accumulator.used:
 *             double_vector_append(accumulator, isotope.abundance * coef)             # <<<<<<<<<<<<<<
 *         else:
 *             printf("Error, unordered isotopes for %s\n", element.symbol)
 */
      (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_accumulator, (__pyx_v_isotope->abundance * __pyx_v_coef)));

      /* "brainpy/_c/isotopic_constants.pyx":122
 *                 double_vector_append(accumulator, 0.)
 *             double_vector_append(accumulator, isotope.abundance * coef)
 *         elif current_order == accumulator.used:             # <<<<<<<<<<<<<<
 *             double_vector_append(accumulator, isotope.abundance * coef)
 *         else:
 */
      goto __pyx_L6;
    }

    /* "brainpy/_c/isotopic_constants.pyx":125
 *             double_vector_append(accumulator, isotope.abundance * coef)
 *         else:
 *             printf("Error, unordered isotopes for %s\n", element.symbol)             # <<<<<<<<<<<<<<
 *     return accumulator
 * 
 */
    /*else*/ {
      (void)(printf(((char const *)"Error, unordered isotopes for %s\n"), __pyx_v_element->symbol));
    }
    __pyx_L6:;
  }

  /* "brainpy/_c/isotopic_constants.pyx":126
 *         else:
 *             printf("Error, unordered isotopes for %s\n", element.symbol)
 *     return accumulator             # <<<<<<<<<<<<<<
 * 
 * cdef PolynomialParameters* make_polynomial_parameters(Element* element, bint with_mass, dvec* accumulator) nogil:
 */
  __pyx_r = __pyx_v_accumulator;
  goto __pyx_L0;

  /* "brainpy/_c/isotopic_constants.pyx":102
 * 
 * 
 * cdef dvec* compute_isotopic_coefficients(Element* element, bint with_mass, dvec* accumulator) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         int max_isotope_number, current_order
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":128
 *     return accumulator
 * 
 * cdef PolynomialParameters* make_polynomial_parameters(Element* element, bint with_mass, dvec* accumulator) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         dvec* elementary_symmetric_polynomial
 */

static struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *__pyx_f_7brainpy_2_c_18isotopic_constants_make_polynomial_parameters(struct __pyx_t_7brainpy_2_c_11composition_Element *__pyx_v_element, int __pyx_v_with_mass, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_accumulator) {
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_elementary_symmetric_polynomial;
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_power_sum;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *__pyx_v_result;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *__pyx_r;

  /* "brainpy/_c/isotopic_constants.pyx":134
 *         PolynomialParameters* result
 * 
 *     compute_isotopic_coefficients(element, with_mass, accumulator)             # <<<<<<<<<<<<<<
 * 
 *     elementary_symmetric_polynomial = vietes(accumulator)
 */
  (void)(__pyx_f_7brainpy_2_c_18isotopic_constants_compute_isotopic_coefficients(__pyx_v_element, __pyx_v_with_mass, __pyx_v_accumulator));

  /* "brainpy/_c/isotopic_constants.pyx":136
 *     compute_isotopic_coefficients(element, with_mass, accumulator)
 * 
 *     elementary_symmetric_polynomial = vietes(accumulator)             # <<<<<<<<<<<<<<
 *     power_sum = make_double_vector_with_size(elementary_symmetric_polynomial.used + 4)
 *     newton(power_sum, elementary_symmetric_polynomial, accumulator.used - 1)
 */
  __pyx_v_elementary_symmetric_polynomial = __pyx_f_7brainpy_2_c_18isotopic_constants_vietes(__pyx_v_accumulator);

  /* "brainpy/_c/isotopic_constants.pyx":137
 * 
 *     elementary_symmetric_polynomial = vietes(accumulator)
 *     power_sum = make_double_vector_with_size(elementary_symmetric_polynomial.used + 4)             # <<<<<<<<<<<<<<
 *     newton(power_sum, elementary_symmetric_polynomial, accumulator.used - 1)
 *     result = <PolynomialParameters*>malloc(sizeof(PolynomialParameters))
 */
  __pyx_v_power_sum = __pyx_f_7brainpy_2_c_13double_vector_make_double_vector_with_size((__pyx_v_elementary_symmetric_polynomial->used + 4));

  /* "brainpy/_c/isotopic_constants.pyx":138
 *     elementary_symmetric_polynomial = vietes(accumulator)
 *     power_sum = make_double_vector_with_size(elementary_symmetric_polynomial.used + 4)
 *     newton(power_sum, elementary_symmetric_polynomial, accumulator.used - 1)             # <<<<<<<<<<<<<<
 *     result = <PolynomialParameters*>malloc(sizeof(PolynomialParameters))
 *     result.elementary_symmetric_polynomial = elementary_symmetric_polynomial
 */
  __pyx_f_7brainpy_2_c_18isotopic_constants_newton(__pyx_v_power_sum, __pyx_v_elementary_symmetric_polynomial, (__pyx_v_accumulator->used - 1));

  /* "brainpy/_c/isotopic_constants.pyx":139
 *     power_sum = make_double_vector_with_size(elementary_symmetric_polynomial.used + 4)
 *     newton(power_sum, elementary_symmetric_polynomial, accumulator.used - 1)
 *     result = <PolynomialParameters*>malloc(sizeof(PolynomialParameters))             # <<<<<<<<<<<<<<
 *     result.elementary_symmetric_polynomial = elementary_symmetric_polynomial
 *     result.power_sum = power_sum
 */
  __pyx_v_result = ((struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *)malloc((sizeof(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters))));

  /* "brainpy/_c/isotopic_constants.pyx":140
 *     newton(power_sum, elementary_symmetric_polynomial, accumulator.used - 1)
 *     result = <PolynomialParameters*>malloc(sizeof(PolynomialParameters))
 *     result.elementary_symmetric_polynomial = elementary_symmetric_polynomial             # <<<<<<<<<<<<<<
 *     result.power_sum = power_sum
 *     return result
 */
  __pyx_v_result->elementary_symmetric_polynomial = __pyx_v_elementary_symmetric_polynomial;

  /* "brainpy/_c/isotopic_constants.pyx":141
 *     result = <PolynomialParameters*>malloc(sizeof(PolynomialParameters))
 *     result.elementary_symmetric_polynomial = elementary_symmetric_polynomial
 *     result.power_sum = power_sum             # <<<<<<<<<<<<<<
 *     return result
 * 
 */
  __pyx_v_result->power_sum = __pyx_v_power_sum;

  /* "brainpy/_c/isotopic_constants.pyx":142
 *     result.elementary_symmetric_polynomial = elementary_symmetric_polynomial
 *     result.power_sum = power_sum
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "brainpy/_c/isotopic_constants.pyx":128
 *     return accumulator
 * 
 * cdef PolynomialParameters* make_polynomial_parameters(Element* element, bint with_mass, dvec* accumulator) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         dvec* elementary_symmetric_polynomial
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":145
 * 
 * 
 * cdef void print_polynomial_parameters(PolynomialParameters* params) nogil:             # <<<<<<<<<<<<<<
 *     printf("PolynomialParameters: %d\n", <int>params)
 *     printf("  ")
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants_print_polynomial_parameters(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *__pyx_v_params) {

  /* "brainpy/_c/isotopic_constants.pyx":146
 * 
 * cdef void print_polynomial_parameters(PolynomialParameters* params) nogil:
 *     printf("PolynomialParameters: %d\n", <int>params)             # <<<<<<<<<<<<<<
 *     printf("  ")
 *     print_double_vector(params.elementary_symmetric_polynomial)
 */
  (void)(printf(((char const *)"PolynomialParameters: %d\n"), ((int)__pyx_v_params)));

  /* "brainpy/_c/isotopic_constants.pyx":147
 * cdef void print_polynomial_parameters(PolynomialParameters* params) nogil:
 *     printf("PolynomialParameters: %d\n", <int>params)
 *     printf("  ")             # <<<<<<<<<<<<<<
 *     print_double_vector(params.elementary_symmetric_polynomial)
 *     printf("  ")
 */
  (void)(printf(((char const *)"  ")));

  /* "brainpy/_c/isotopic_constants.pyx":148
 *     printf("PolynomialParameters: %d\n", <int>params)
 *     printf("  ")
 *     print_double_vector(params.elementary_symmetric_polynomial)             # <<<<<<<<<<<<<<
 *     printf("  ")
 *     print_double_vector(params.power_sum)
 */
  __pyx_f_7brainpy_2_c_13double_vector_print_double_vector(__pyx_v_params->elementary_symmetric_polynomial);

  /* "brainpy/_c/isotopic_constants.pyx":149
 *     printf("  ")
 *     print_double_vector(params.elementary_symmetric_polynomial)
 *     printf("  ")             # <<<<<<<<<<<<<<
 *     print_double_vector(params.power_sum)
 *     printf("\n")
 */
  (void)(printf(((char const *)"  ")));

  /* "brainpy/_c/isotopic_constants.pyx":150
 *     print_double_vector(params.elementary_symmetric_polynomial)
 *     printf("  ")
 *     print_double_vector(params.power_sum)             # <<<<<<<<<<<<<<
 *     printf("\n")
 * 
 */
  __pyx_f_7brainpy_2_c_13double_vector_print_double_vector(__pyx_v_params->power_sum);

  /* "brainpy/_c/isotopic_constants.pyx":151
 *     printf("  ")
 *     print_double_vector(params.power_sum)
 *     printf("\n")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (void)(printf(((char const *)"\n")));

  /* "brainpy/_c/isotopic_constants.pyx":145
 * 
 * 
 * cdef void print_polynomial_parameters(PolynomialParameters* params) nogil:             # <<<<<<<<<<<<<<
 *     printf("PolynomialParameters: %d\n", <int>params)
 *     printf("  ")
 */

  /* function exit code */
}

/* "brainpy/_c/isotopic_constants.pyx":154
 * 
 * 
 * cdef void free_polynomial_parameters(PolynomialParameters* params) nogil:             # <<<<<<<<<<<<<<
 *     free_double_vector(params.elementary_symmetric_polynomial)
 *     free_double_vector(params.power_sum)
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants_free_polynomial_parameters(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *__pyx_v_params) {

  /* "brainpy/_c/isotopic_constants.pyx":155
 * 
 * cdef void free_polynomial_parameters(PolynomialParameters* params) nogil:
 *     free_double_vector(params.elementary_symmetric_polynomial)             # <<<<<<<<<<<<<<
 *     free_double_vector(params.power_sum)
 *     free(params)
 */
  __pyx_f_7brainpy_2_c_13double_vector_free_double_vector(__pyx_v_params->elementary_symmetric_polynomial);

  /* "brainpy/_c/isotopic_constants.pyx":156
 * cdef void free_polynomial_parameters(PolynomialParameters* params) nogil:
 *     free_double_vector(params.elementary_symmetric_polynomial)
 *     free_double_vector(params.power_sum)             # <<<<<<<<<<<<<<
 *     free(params)
 * 
 */
  __pyx_f_7brainpy_2_c_13double_vector_free_double_vector(__pyx_v_params->power_sum);

  /* "brainpy/_c/isotopic_constants.pyx":157
 *     free_double_vector(params.elementary_symmetric_polynomial)
 *     free_double_vector(params.power_sum)
 *     free(params)             # <<<<<<<<<<<<<<
 * 
 * # -----------------------------------------------------------------------------
 */
  free(__pyx_v_params);

  /* "brainpy/_c/isotopic_constants.pyx":154
 * 
 * 
 * cdef void free_polynomial_parameters(PolynomialParameters* params) nogil:             # <<<<<<<<<<<<<<
 *     free_double_vector(params.elementary_symmetric_polynomial)
 *     free_double_vector(params.power_sum)
 */

  /* function exit code */
}

/* "brainpy/_c/isotopic_constants.pyx":163
 * 
 * 
 * cdef void print_phi_constants(PhiConstants* constants) nogil:             # <<<<<<<<<<<<<<
 *     printf("PhiConstants: %d\n", <int>constants)
 *     printf("Element: %s, Order: %d\n", constants.element.symbol, constants.order)
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants_print_phi_constants(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *__pyx_v_constants) {

  /* "brainpy/_c/isotopic_constants.pyx":164
 * 
 * cdef void print_phi_constants(PhiConstants* constants) nogil:
 *     printf("PhiConstants: %d\n", <int>constants)             # <<<<<<<<<<<<<<
 *     printf("Element: %s, Order: %d\n", constants.element.symbol, constants.order)
 *     printf("Element Coefficients:\n")
 */
  (void)(printf(((char const *)"PhiConstants: %d\n"), ((int)__pyx_v_constants)));

  /* "brainpy/_c/isotopic_constants.pyx":165
 * cdef void print_phi_constants(PhiConstants* constants) nogil:
 *     printf("PhiConstants: %d\n", <int>constants)
 *     printf("Element: %s, Order: %d\n", constants.element.symbol, constants.order)             # <<<<<<<<<<<<<<
 *     printf("Element Coefficients:\n")
 *     print_polynomial_parameters(constants.element_coefficients)
 */
  (void)(printf(((char const *)"Element: %s, Order: %d\n"), __pyx_v_constants->element->symbol, __pyx_v_constants->order));

  /* "brainpy/_c/isotopic_constants.pyx":166
 *     printf("PhiConstants: %d\n", <int>constants)
 *     printf("Element: %s, Order: %d\n", constants.element.symbol, constants.order)
 *     printf("Element Coefficients:\n")             # <<<<<<<<<<<<<<
 *     print_polynomial_parameters(constants.element_coefficients)
 *     printf("Mass Coefficients:\n")
 */
  (void)(printf(((char const *)"Element Coefficients:\n")));

  /* "brainpy/_c/isotopic_constants.pyx":167
 *     printf("Element: %s, Order: %d\n", constants.element.symbol, constants.order)
 *     printf("Element Coefficients:\n")
 *     print_polynomial_parameters(constants.element_coefficients)             # <<<<<<<<<<<<<<
 *     printf("Mass Coefficients:\n")
 *     print_polynomial_parameters(constants.mass_coefficients)
 */
  __pyx_f_7brainpy_2_c_18isotopic_constants_print_polynomial_parameters(__pyx_v_constants->element_coefficients);

  /* "brainpy/_c/isotopic_constants.pyx":168
 *     printf("Element Coefficients:\n")
 *     print_polynomial_parameters(constants.element_coefficients)
 *     printf("Mass Coefficients:\n")             # <<<<<<<<<<<<<<
 *     print_polynomial_parameters(constants.mass_coefficients)
 *     printf("\n")
 */
  (void)(printf(((char const *)"Mass Coefficients:\n")));

  /* "brainpy/_c/isotopic_constants.pyx":169
 *     print_polynomial_parameters(constants.element_coefficients)
 *     printf("Mass Coefficients:\n")
 *     print_polynomial_parameters(constants.mass_coefficients)             # <<<<<<<<<<<<<<
 *     printf("\n")
 * 
 */
  __pyx_f_7brainpy_2_c_18isotopic_constants_print_polynomial_parameters(__pyx_v_constants->mass_coefficients);

  /* "brainpy/_c/isotopic_constants.pyx":170
 *     printf("Mass Coefficients:\n")
 *     print_polynomial_parameters(constants.mass_coefficients)
 *     printf("\n")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (void)(printf(((char const *)"\n")));

  /* "brainpy/_c/isotopic_constants.pyx":163
 * 
 * 
 * cdef void print_phi_constants(PhiConstants* constants) nogil:             # <<<<<<<<<<<<<<
 *     printf("PhiConstants: %d\n", <int>constants)
 *     printf("Element: %s, Order: %d\n", constants.element.symbol, constants.order)
 */

  /* function exit code */
}

/* "brainpy/_c/isotopic_constants.pyx":173
 * 
 * 
 * cdef void free_phi_constants(PhiConstants* constants) nogil:             # <<<<<<<<<<<<<<
 *     free_polynomial_parameters(constants.element_coefficients)
 *     free_polynomial_parameters(constants.mass_coefficients)
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants_free_phi_constants(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *__pyx_v_constants) {

  /* "brainpy/_c/isotopic_constants.pyx":174
 * 
 * cdef void free_phi_constants(PhiConstants* constants) nogil:
 *     free_polynomial_parameters(constants.element_coefficients)             # <<<<<<<<<<<<<<
 *     free_polynomial_parameters(constants.mass_coefficients)
 *     free(constants)
 */
  __pyx_f_7brainpy_2_c_18isotopic_constants_free_polynomial_parameters(__pyx_v_constants->element_coefficients);

  /* "brainpy/_c/isotopic_constants.pyx":175
 * cdef void free_phi_constants(PhiConstants* constants) nogil:
 *     free_polynomial_parameters(constants.element_coefficients)
 *     free_polynomial_parameters(constants.mass_coefficients)             # <<<<<<<<<<<<<<
 *     free(constants)
 * 
 */
  __pyx_f_7brainpy_2_c_18isotopic_constants_free_polynomial_parameters(__pyx_v_constants->mass_coefficients);

  /* "brainpy/_c/isotopic_constants.pyx":176
 *     free_polynomial_parameters(constants.element_coefficients)
 *     free_polynomial_parameters(constants.mass_coefficients)
 *     free(constants)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_constants);

  /* "brainpy/_c/isotopic_constants.pyx":173
 * 
 * 
 * cdef void free_phi_constants(PhiConstants* constants) nogil:             # <<<<<<<<<<<<<<
 *     free_polynomial_parameters(constants.element_coefficients)
 *     free_polynomial_parameters(constants.mass_coefficients)
 */

  /* function exit code */
}

/* "brainpy/_c/isotopic_constants.pyx":185
 * 
 * 
 * cdef IsotopicConstants* make_isotopic_constants() nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         IsotopicConstants* result
 */

static struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_f_7brainpy_2_c_18isotopic_constants_make_isotopic_constants(void) {
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_result;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_r;

  /* "brainpy/_c/isotopic_constants.pyx":188
 *     cdef:
 *         IsotopicConstants* result
 *     result = <IsotopicConstants*>malloc(sizeof(IsotopicConstants))             # <<<<<<<<<<<<<<
 *     result.constants = <PhiConstants**>malloc(sizeof(PhiConstants*) * DEFAULT_ISOTOPIC_CONSTANTS_SIZE)
 *     result.size = DEFAULT_ISOTOPIC_CONSTANTS_SIZE
 */
  __pyx_v_result = ((struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *)malloc((sizeof(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants))));

  /* "brainpy/_c/isotopic_constants.pyx":189
 *         IsotopicConstants* result
 *     result = <IsotopicConstants*>malloc(sizeof(IsotopicConstants))
 *     result.constants = <PhiConstants**>malloc(sizeof(PhiConstants*) * DEFAULT_ISOTOPIC_CONSTANTS_SIZE)             # <<<<<<<<<<<<<<
 *     result.size = DEFAULT_ISOTOPIC_CONSTANTS_SIZE
 *     result.used = 0
 */
  __pyx_v_result->constants = ((struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants **)malloc(((sizeof(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *)) * __pyx_v_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE)));

  /* "brainpy/_c/isotopic_constants.pyx":190
 *     result = <IsotopicConstants*>malloc(sizeof(IsotopicConstants))
 *     result.constants = <PhiConstants**>malloc(sizeof(PhiConstants*) * DEFAULT_ISOTOPIC_CONSTANTS_SIZE)
 *     result.size = DEFAULT_ISOTOPIC_CONSTANTS_SIZE             # <<<<<<<<<<<<<<
 *     result.used = 0
 *     return result
 */
  __pyx_v_result->size = __pyx_v_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE;

  /* "brainpy/_c/isotopic_constants.pyx":191
 *     result.constants = <PhiConstants**>malloc(sizeof(PhiConstants*) * DEFAULT_ISOTOPIC_CONSTANTS_SIZE)
 *     result.size = DEFAULT_ISOTOPIC_CONSTANTS_SIZE
 *     result.used = 0             # <<<<<<<<<<<<<<
 *     return result
 * 
 */
  __pyx_v_result->used = 0;

  /* "brainpy/_c/isotopic_constants.pyx":192
 *     result.size = DEFAULT_ISOTOPIC_CONSTANTS_SIZE
 *     result.used = 0
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "brainpy/_c/isotopic_constants.pyx":185
 * 
 * 
 * cdef IsotopicConstants* make_isotopic_constants() nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         IsotopicConstants* result
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":195
 * 
 * 
 * cdef int isotopic_constants_resize(IsotopicConstants* ics) nogil:             # <<<<<<<<<<<<<<
 *     ics.constants = <PhiConstants**>realloc(ics.constants, sizeof(PhiConstants*) * ics.size * 2)
 *     ics.size *= 2
 */

static int __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_resize(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_ics) {
  int __pyx_r;
  int __pyx_t_1;

  /* "brainpy/_c/isotopic_constants.pyx":196
 * 
 * cdef int isotopic_constants_resize(IsotopicConstants* ics) nogil:
 *     ics.constants = <PhiConstants**>realloc(ics.constants, sizeof(PhiConstants*) * ics.size * 2)             # <<<<<<<<<<<<<<
 *     ics.size *= 2
 *     if ics.constants == NULL:
 */
  __pyx_v_ics->constants = ((struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants **)realloc(__pyx_v_ics->constants, (((sizeof(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *)) * __pyx_v_ics->size) * 2)));

  /* "brainpy/_c/isotopic_constants.pyx":197
 * cdef int isotopic_constants_resize(IsotopicConstants* ics) nogil:
 *     ics.constants = <PhiConstants**>realloc(ics.constants, sizeof(PhiConstants*) * ics.size * 2)
 *     ics.size *= 2             # <<<<<<<<<<<<<<
 *     if ics.constants == NULL:
 *         return -1
 */
  __pyx_v_ics->size = (__pyx_v_ics->size * 2);

  /* "brainpy/_c/isotopic_constants.pyx":198
 *     ics.constants = <PhiConstants**>realloc(ics.constants, sizeof(PhiConstants*) * ics.size * 2)
 *     ics.size *= 2
 *     if ics.constants == NULL:             # <<<<<<<<<<<<<<
 *         return -1
 *     return 0
 */
  __pyx_t_1 = ((__pyx_v_ics->constants == NULL) != 0);
  if (__pyx_t_1) {

    /* "brainpy/_c/isotopic_constants.pyx":199
 *     ics.size *= 2
 *     if ics.constants == NULL:
 *         return -1             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "brainpy/_c/isotopic_constants.pyx":198
 *     ics.constants = <PhiConstants**>realloc(ics.constants, sizeof(PhiConstants*) * ics.size * 2)
 *     ics.size *= 2
 *     if ics.constants == NULL:             # <<<<<<<<<<<<<<
 *         return -1
 *     return 0
 */
  }

  /* "brainpy/_c/isotopic_constants.pyx":200
 *     if ics.constants == NULL:
 *         return -1
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "brainpy/_c/isotopic_constants.pyx":195
 * 
 * 
 * cdef int isotopic_constants_resize(IsotopicConstants* ics) nogil:             # <<<<<<<<<<<<<<
 *     ics.constants = <PhiConstants**>realloc(ics.constants, sizeof(PhiConstants*) * ics.size * 2)
 *     ics.size *= 2
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":203
 * 
 * 
 * cdef void free_isotopic_constants(IsotopicConstants* isotopes) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants_free_isotopic_constants(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_isotopes) {
  size_t __pyx_v_i;
  int __pyx_t_1;

  /* "brainpy/_c/isotopic_constants.pyx":207
 *         size_t i
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     while i < isotopes.used:
 *         free_phi_constants(isotopes.constants[i])
 */
  __pyx_v_i = 0;

  /* "brainpy/_c/isotopic_constants.pyx":208
 * 
 *     i = 0
 *     while i < isotopes.used:             # <<<<<<<<<<<<<<
 *         free_phi_constants(isotopes.constants[i])
 *         i += 1
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_i < __pyx_v_isotopes->used) != 0);
    if (!__pyx_t_1) break;

    /* "brainpy/_c/isotopic_constants.pyx":209
 *     i = 0
 *     while i < isotopes.used:
 *         free_phi_constants(isotopes.constants[i])             # <<<<<<<<<<<<<<
 *         i += 1
 *     free(isotopes.constants)
 */
    __pyx_f_7brainpy_2_c_18isotopic_constants_free_phi_constants((__pyx_v_isotopes->constants[__pyx_v_i]));

    /* "brainpy/_c/isotopic_constants.pyx":210
 *     while i < isotopes.used:
 *         free_phi_constants(isotopes.constants[i])
 *         i += 1             # <<<<<<<<<<<<<<
 *     free(isotopes.constants)
 *     free(isotopes)
 */
    __pyx_v_i = (__pyx_v_i + 1);
  }

  /* "brainpy/_c/isotopic_constants.pyx":211
 *         free_phi_constants(isotopes.constants[i])
 *         i += 1
 *     free(isotopes.constants)             # <<<<<<<<<<<<<<
 *     free(isotopes)
 * 
 */
  free(__pyx_v_isotopes->constants);

  /* "brainpy/_c/isotopic_constants.pyx":212
 *         i += 1
 *     free(isotopes.constants)
 *     free(isotopes)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free(__pyx_v_isotopes);

  /* "brainpy/_c/isotopic_constants.pyx":203
 * 
 * 
 * cdef void free_isotopic_constants(IsotopicConstants* isotopes) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

  /* function exit code */
}

/* "brainpy/_c/isotopic_constants.pyx":215
 * 
 * 
 * cdef void isotopic_constants_add_element(IsotopicConstants* isotopes, char* element_symbol) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         Element* element
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_add_element(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_isotopes, char *__pyx_v_element_symbol) {
  struct __pyx_t_7brainpy_2_c_11composition_Element *__pyx_v_element;
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *__pyx_v_accumulator;
  int __pyx_v_order;
  int __pyx_v_status;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *__pyx_v_element_parameters;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *__pyx_v_mass_parameters;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *__pyx_v_phi_constants;
  int __pyx_t_1;

  /* "brainpy/_c/isotopic_constants.pyx":225
 *         char* element_buffer
 * 
 *     status = isotopic_constants_get(isotopes, element_symbol, &phi_constants)             # <<<<<<<<<<<<<<
 *     if status == 0:
 *         return
 */
  __pyx_v_status = __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get(__pyx_v_isotopes, __pyx_v_element_symbol, (&__pyx_v_phi_constants));

  /* "brainpy/_c/isotopic_constants.pyx":226
 * 
 *     status = isotopic_constants_get(isotopes, element_symbol, &phi_constants)
 *     if status == 0:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_1 = ((__pyx_v_status == 0) != 0);
  if (__pyx_t_1) {

    /* "brainpy/_c/isotopic_constants.pyx":227
 *     status = isotopic_constants_get(isotopes, element_symbol, &phi_constants)
 *     if status == 0:
 *         return             # <<<<<<<<<<<<<<
 * 
 *     phi_constants = NULL
 */
    goto __pyx_L0;

    /* "brainpy/_c/isotopic_constants.pyx":226
 * 
 *     status = isotopic_constants_get(isotopes, element_symbol, &phi_constants)
 *     if status == 0:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  }

  /* "brainpy/_c/isotopic_constants.pyx":229
 *         return
 * 
 *     phi_constants = NULL             # <<<<<<<<<<<<<<
 * 
 *     status = element_hash_table_get(_ElementTable, element_symbol, &element)
 */
  __pyx_v_phi_constants = NULL;

  /* "brainpy/_c/isotopic_constants.pyx":231
 *     phi_constants = NULL
 * 
 *     status = element_hash_table_get(_ElementTable, element_symbol, &element)             # <<<<<<<<<<<<<<
 *     if status == -1:
 *         printf("Could not resolve element_symbol %s\n", element_symbol)
 */
  __pyx_v_status = __pyx_f_7brainpy_2_c_11composition_element_hash_table_get(__pyx_v_7brainpy_2_c_11composition__ElementTable, __pyx_v_element_symbol, (&__pyx_v_element));

  /* "brainpy/_c/isotopic_constants.pyx":232
 * 
 *     status = element_hash_table_get(_ElementTable, element_symbol, &element)
 *     if status == -1:             # <<<<<<<<<<<<<<
 *         printf("Could not resolve element_symbol %s\n", element_symbol)
 *         return
 */
  __pyx_t_1 = ((__pyx_v_status == -1L) != 0);
  if (__pyx_t_1) {

    /* "brainpy/_c/isotopic_constants.pyx":233
 *     status = element_hash_table_get(_ElementTable, element_symbol, &element)
 *     if status == -1:
 *         printf("Could not resolve element_symbol %s\n", element_symbol)             # <<<<<<<<<<<<<<
 *         return
 *     accumulator = make_double_vector()
 */
    (void)(printf(((char const *)"Could not resolve element_symbol %s\n"), __pyx_v_element_symbol));

    /* "brainpy/_c/isotopic_constants.pyx":234
 *     if status == -1:
 *         printf("Could not resolve element_symbol %s\n", element_symbol)
 *         return             # <<<<<<<<<<<<<<
 *     accumulator = make_double_vector()
 *     order = element_max_neutron_shift(element)
 */
    goto __pyx_L0;

    /* "brainpy/_c/isotopic_constants.pyx":232
 * 
 *     status = element_hash_table_get(_ElementTable, element_symbol, &element)
 *     if status == -1:             # <<<<<<<<<<<<<<
 *         printf("Could not resolve element_symbol %s\n", element_symbol)
 *         return
 */
  }

  /* "brainpy/_c/isotopic_constants.pyx":235
 *         printf("Could not resolve element_symbol %s\n", element_symbol)
 *         return
 *     accumulator = make_double_vector()             # <<<<<<<<<<<<<<
 *     order = element_max_neutron_shift(element)
 *     element_parameters = make_polynomial_parameters(element, False, accumulator)
 */
  __pyx_v_accumulator = __pyx_f_7brainpy_2_c_13double_vector_make_double_vector();

  /* "brainpy/_c/isotopic_constants.pyx":236
 *         return
 *     accumulator = make_double_vector()
 *     order = element_max_neutron_shift(element)             # <<<<<<<<<<<<<<
 *     element_parameters = make_polynomial_parameters(element, False, accumulator)
 *     reset_double_vector(accumulator)
 */
  __pyx_v_order = __pyx_f_7brainpy_2_c_11composition_element_max_neutron_shift(__pyx_v_element);

  /* "brainpy/_c/isotopic_constants.pyx":237
 *     accumulator = make_double_vector()
 *     order = element_max_neutron_shift(element)
 *     element_parameters = make_polynomial_parameters(element, False, accumulator)             # <<<<<<<<<<<<<<
 *     reset_double_vector(accumulator)
 *     mass_parameters = make_polynomial_parameters(element, True, accumulator)
 */
  __pyx_v_element_parameters = __pyx_f_7brainpy_2_c_18isotopic_constants_make_polynomial_parameters(__pyx_v_element, 0, __pyx_v_accumulator);

  /* "brainpy/_c/isotopic_constants.pyx":238
 *     order = element_max_neutron_shift(element)
 *     element_parameters = make_polynomial_parameters(element, False, accumulator)
 *     reset_double_vector(accumulator)             # <<<<<<<<<<<<<<
 *     mass_parameters = make_polynomial_parameters(element, True, accumulator)
 *     free_double_vector(accumulator)
 */
  __pyx_f_7brainpy_2_c_13double_vector_reset_double_vector(__pyx_v_accumulator);

  /* "brainpy/_c/isotopic_constants.pyx":239
 *     element_parameters = make_polynomial_parameters(element, False, accumulator)
 *     reset_double_vector(accumulator)
 *     mass_parameters = make_polynomial_parameters(element, True, accumulator)             # <<<<<<<<<<<<<<
 *     free_double_vector(accumulator)
 *     phi_constants = <PhiConstants*>malloc(sizeof(PhiConstants))
 */
  __pyx_v_mass_parameters = __pyx_f_7brainpy_2_c_18isotopic_constants_make_polynomial_parameters(__pyx_v_element, 1, __pyx_v_accumulator);

  /* "brainpy/_c/isotopic_constants.pyx":240
 *     reset_double_vector(accumulator)
 *     mass_parameters = make_polynomial_parameters(element, True, accumulator)
 *     free_double_vector(accumulator)             # <<<<<<<<<<<<<<
 *     phi_constants = <PhiConstants*>malloc(sizeof(PhiConstants))
 *     phi_constants.order = order
 */
  __pyx_f_7brainpy_2_c_13double_vector_free_double_vector(__pyx_v_accumulator);

  /* "brainpy/_c/isotopic_constants.pyx":241
 *     mass_parameters = make_polynomial_parameters(element, True, accumulator)
 *     free_double_vector(accumulator)
 *     phi_constants = <PhiConstants*>malloc(sizeof(PhiConstants))             # <<<<<<<<<<<<<<
 *     phi_constants.order = order
 *     phi_constants.element = element
 */
  __pyx_v_phi_constants = ((struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *)malloc((sizeof(struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants))));

  /* "brainpy/_c/isotopic_constants.pyx":242
 *     free_double_vector(accumulator)
 *     phi_constants = <PhiConstants*>malloc(sizeof(PhiConstants))
 *     phi_constants.order = order             # <<<<<<<<<<<<<<
 *     phi_constants.element = element
 *     phi_constants.element_coefficients = element_parameters
 */
  __pyx_v_phi_constants->order = __pyx_v_order;

  /* "brainpy/_c/isotopic_constants.pyx":243
 *     phi_constants = <PhiConstants*>malloc(sizeof(PhiConstants))
 *     phi_constants.order = order
 *     phi_constants.element = element             # <<<<<<<<<<<<<<
 *     phi_constants.element_coefficients = element_parameters
 *     phi_constants.mass_coefficients = mass_parameters
 */
  __pyx_v_phi_constants->element = __pyx_v_element;

  /* "brainpy/_c/isotopic_constants.pyx":244
 *     phi_constants.order = order
 *     phi_constants.element = element
 *     phi_constants.element_coefficients = element_parameters             # <<<<<<<<<<<<<<
 *     phi_constants.mass_coefficients = mass_parameters
 *     if isotopes.used + 1 == isotopes.size:
 */
  __pyx_v_phi_constants->element_coefficients = __pyx_v_element_parameters;

  /* "brainpy/_c/isotopic_constants.pyx":245
 *     phi_constants.element = element
 *     phi_constants.element_coefficients = element_parameters
 *     phi_constants.mass_coefficients = mass_parameters             # <<<<<<<<<<<<<<
 *     if isotopes.used + 1 == isotopes.size:
 *         isotopic_constants_resize(isotopes)
 */
  __pyx_v_phi_constants->mass_coefficients = __pyx_v_mass_parameters;

  /* "brainpy/_c/isotopic_constants.pyx":246
 *     phi_constants.element_coefficients = element_parameters
 *     phi_constants.mass_coefficients = mass_parameters
 *     if isotopes.used + 1 == isotopes.size:             # <<<<<<<<<<<<<<
 *         isotopic_constants_resize(isotopes)
 *     isotopes.constants[isotopes.used] = phi_constants
 */
  __pyx_t_1 = (((__pyx_v_isotopes->used + 1) == __pyx_v_isotopes->size) != 0);
  if (__pyx_t_1) {

    /* "brainpy/_c/isotopic_constants.pyx":247
 *     phi_constants.mass_coefficients = mass_parameters
 *     if isotopes.used + 1 == isotopes.size:
 *         isotopic_constants_resize(isotopes)             # <<<<<<<<<<<<<<
 *     isotopes.constants[isotopes.used] = phi_constants
 *     isotopes.used += 1
 */
    (void)(__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_resize(__pyx_v_isotopes));

    /* "brainpy/_c/isotopic_constants.pyx":246
 *     phi_constants.element_coefficients = element_parameters
 *     phi_constants.mass_coefficients = mass_parameters
 *     if isotopes.used + 1 == isotopes.size:             # <<<<<<<<<<<<<<
 *         isotopic_constants_resize(isotopes)
 *     isotopes.constants[isotopes.used] = phi_constants
 */
  }

  /* "brainpy/_c/isotopic_constants.pyx":248
 *     if isotopes.used + 1 == isotopes.size:
 *         isotopic_constants_resize(isotopes)
 *     isotopes.constants[isotopes.used] = phi_constants             # <<<<<<<<<<<<<<
 *     isotopes.used += 1
 * 
 */
  (__pyx_v_isotopes->constants[__pyx_v_isotopes->used]) = __pyx_v_phi_constants;

  /* "brainpy/_c/isotopic_constants.pyx":249
 *         isotopic_constants_resize(isotopes)
 *     isotopes.constants[isotopes.used] = phi_constants
 *     isotopes.used += 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_isotopes->used = (__pyx_v_isotopes->used + 1);

  /* "brainpy/_c/isotopic_constants.pyx":215
 * 
 * 
 * cdef void isotopic_constants_add_element(IsotopicConstants* isotopes, char* element_symbol) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         Element* element
 */

  /* function exit code */
  __pyx_L0:;
}

/* "brainpy/_c/isotopic_constants.pyx":252
 * 
 * 
 * cdef int isotopic_constants_get(IsotopicConstants* isotopes, char* element_symbol, PhiConstants** out) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

static int __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_isotopes, char *__pyx_v_element_symbol, struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants **__pyx_v_out) {
  size_t __pyx_v_i;
  int __pyx_r;
  int __pyx_t_1;

  /* "brainpy/_c/isotopic_constants.pyx":257
 *         int status
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     while i < isotopes.used:
 *         if strcmp(isotopes.constants[i].element.symbol, element_symbol) == 0:
 */
  __pyx_v_i = 0;

  /* "brainpy/_c/isotopic_constants.pyx":258
 * 
 *     i = 0
 *     while i < isotopes.used:             # <<<<<<<<<<<<<<
 *         if strcmp(isotopes.constants[i].element.symbol, element_symbol) == 0:
 *             out[0] = (isotopes.constants[i])
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_i < __pyx_v_isotopes->used) != 0);
    if (!__pyx_t_1) break;

    /* "brainpy/_c/isotopic_constants.pyx":259
 *     i = 0
 *     while i < isotopes.used:
 *         if strcmp(isotopes.constants[i].element.symbol, element_symbol) == 0:             # <<<<<<<<<<<<<<
 *             out[0] = (isotopes.constants[i])
 *             return 0
 */
    __pyx_t_1 = ((strcmp((__pyx_v_isotopes->constants[__pyx_v_i])->element->symbol, __pyx_v_element_symbol) == 0) != 0);
    if (__pyx_t_1) {

      /* "brainpy/_c/isotopic_constants.pyx":260
 *     while i < isotopes.used:
 *         if strcmp(isotopes.constants[i].element.symbol, element_symbol) == 0:
 *             out[0] = (isotopes.constants[i])             # <<<<<<<<<<<<<<
 *             return 0
 *         i += 1
 */
      (__pyx_v_out[0]) = (__pyx_v_isotopes->constants[__pyx_v_i]);

      /* "brainpy/_c/isotopic_constants.pyx":261
 *         if strcmp(isotopes.constants[i].element.symbol, element_symbol) == 0:
 *             out[0] = (isotopes.constants[i])
 *             return 0             # <<<<<<<<<<<<<<
 *         i += 1
 *     return 1
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "brainpy/_c/isotopic_constants.pyx":259
 *     i = 0
 *     while i < isotopes.used:
 *         if strcmp(isotopes.constants[i].element.symbol, element_symbol) == 0:             # <<<<<<<<<<<<<<
 *             out[0] = (isotopes.constants[i])
 *             return 0
 */
    }

    /* "brainpy/_c/isotopic_constants.pyx":262
 *             out[0] = (isotopes.constants[i])
 *             return 0
 *         i += 1             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
    __pyx_v_i = (__pyx_v_i + 1);
  }

  /* "brainpy/_c/isotopic_constants.pyx":263
 *             return 0
 *         i += 1
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "brainpy/_c/isotopic_constants.pyx":252
 * 
 * 
 * cdef int isotopic_constants_get(IsotopicConstants* isotopes, char* element_symbol, PhiConstants** out) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":266
 * 
 * 
 * cdef int isotopic_constants_get_by_index(IsotopicConstants* isotopes, size_t index, PhiConstants** out) nogil:             # <<<<<<<<<<<<<<
 *     if index > isotopes.used:
 *         return 1
 */

static int __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get_by_index(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_isotopes, size_t __pyx_v_index, struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants **__pyx_v_out) {
  int __pyx_r;
  int __pyx_t_1;

  /* "brainpy/_c/isotopic_constants.pyx":267
 * 
 * cdef int isotopic_constants_get_by_index(IsotopicConstants* isotopes, size_t index, PhiConstants** out) nogil:
 *     if index > isotopes.used:             # <<<<<<<<<<<<<<
 *         return 1
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_index > __pyx_v_isotopes->used) != 0);
  if (__pyx_t_1) {

    /* "brainpy/_c/isotopic_constants.pyx":268
 * cdef int isotopic_constants_get_by_index(IsotopicConstants* isotopes, size_t index, PhiConstants** out) nogil:
 *     if index > isotopes.used:
 *         return 1             # <<<<<<<<<<<<<<
 *     else:
 *         out[0] = isotopes.constants[index]
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "brainpy/_c/isotopic_constants.pyx":267
 * 
 * cdef int isotopic_constants_get_by_index(IsotopicConstants* isotopes, size_t index, PhiConstants** out) nogil:
 *     if index > isotopes.used:             # <<<<<<<<<<<<<<
 *         return 1
 *     else:
 */
  }

  /* "brainpy/_c/isotopic_constants.pyx":270
 *         return 1
 *     else:
 *         out[0] = isotopes.constants[index]             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  /*else*/ {
    (__pyx_v_out[0]) = (__pyx_v_isotopes->constants[__pyx_v_index]);

    /* "brainpy/_c/isotopic_constants.pyx":271
 *     else:
 *         out[0] = isotopes.constants[index]
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "brainpy/_c/isotopic_constants.pyx":266
 * 
 * 
 * cdef int isotopic_constants_get_by_index(IsotopicConstants* isotopes, size_t index, PhiConstants** out) nogil:             # <<<<<<<<<<<<<<
 *     if index > isotopes.used:
 *         return 1
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":274
 * 
 * 
 * cdef void isotopic_constants_update_coefficients(IsotopicConstants* isotopes) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_update_coefficients(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_isotopes) {
  size_t __pyx_v_i;
  CYTHON_UNUSED size_t __pyx_v_j;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *__pyx_v_constants;
  size_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  size_t __pyx_t_7;

  /* "brainpy/_c/isotopic_constants.pyx":279
 *         PhiConstants* constants
 * 
 *     for i in range(isotopes.used):             # <<<<<<<<<<<<<<
 *         constants = isotopes.constants[i]
 * 
 */
  __pyx_t_1 = __pyx_v_isotopes->used;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "brainpy/_c/isotopic_constants.pyx":280
 * 
 *     for i in range(isotopes.used):
 *         constants = isotopes.constants[i]             # <<<<<<<<<<<<<<
 * 
 *         if isotopes.order < constants.order:
 */
    __pyx_v_constants = (__pyx_v_isotopes->constants[__pyx_v_i]);

    /* "brainpy/_c/isotopic_constants.pyx":282
 *         constants = isotopes.constants[i]
 * 
 *         if isotopes.order < constants.order:             # <<<<<<<<<<<<<<
 *             continue
 * 
 */
    __pyx_t_4 = ((__pyx_v_isotopes->order < __pyx_v_constants->order) != 0);
    if (__pyx_t_4) {

      /* "brainpy/_c/isotopic_constants.pyx":283
 * 
 *         if isotopes.order < constants.order:
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         for j in range(constants.order, isotopes.order + 1):
 */
      goto __pyx_L3_continue;

      /* "brainpy/_c/isotopic_constants.pyx":282
 *         constants = isotopes.constants[i]
 * 
 *         if isotopes.order < constants.order:             # <<<<<<<<<<<<<<
 *             continue
 * 
 */
    }

    /* "brainpy/_c/isotopic_constants.pyx":285
 *             continue
 * 
 *         for j in range(constants.order, isotopes.order + 1):             # <<<<<<<<<<<<<<
 *             double_vector_append(constants.element_coefficients.elementary_symmetric_polynomial, 0.)
 *             double_vector_append(constants.mass_coefficients.elementary_symmetric_polynomial, 0.)
 */
    __pyx_t_5 = (__pyx_v_isotopes->order + 1);
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = __pyx_v_constants->order; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_j = __pyx_t_7;

      /* "brainpy/_c/isotopic_constants.pyx":286
 * 
 *         for j in range(constants.order, isotopes.order + 1):
 *             double_vector_append(constants.element_coefficients.elementary_symmetric_polynomial, 0.)             # <<<<<<<<<<<<<<
 *             double_vector_append(constants.mass_coefficients.elementary_symmetric_polynomial, 0.)
 * 
 */
      (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_constants->element_coefficients->elementary_symmetric_polynomial, 0.));

      /* "brainpy/_c/isotopic_constants.pyx":287
 *         for j in range(constants.order, isotopes.order + 1):
 *             double_vector_append(constants.element_coefficients.elementary_symmetric_polynomial, 0.)
 *             double_vector_append(constants.mass_coefficients.elementary_symmetric_polynomial, 0.)             # <<<<<<<<<<<<<<
 * 
 *         constants.order = constants.element_coefficients.elementary_symmetric_polynomial.used
 */
      (void)(__pyx_f_7brainpy_2_c_13double_vector_double_vector_append(__pyx_v_constants->mass_coefficients->elementary_symmetric_polynomial, 0.));
    }

    /* "brainpy/_c/isotopic_constants.pyx":289
 *             double_vector_append(constants.mass_coefficients.elementary_symmetric_polynomial, 0.)
 * 
 *         constants.order = constants.element_coefficients.elementary_symmetric_polynomial.used             # <<<<<<<<<<<<<<
 * 
 *         newton(constants.element_coefficients.power_sum,
 */
    __pyx_t_7 = __pyx_v_constants->element_coefficients->elementary_symmetric_polynomial->used;
    __pyx_v_constants->order = __pyx_t_7;

    /* "brainpy/_c/isotopic_constants.pyx":291
 *         constants.order = constants.element_coefficients.elementary_symmetric_polynomial.used
 * 
 *         newton(constants.element_coefficients.power_sum,             # <<<<<<<<<<<<<<
 *                constants.element_coefficients.elementary_symmetric_polynomial,
 *                constants.order)
 */
    __pyx_f_7brainpy_2_c_18isotopic_constants_newton(__pyx_v_constants->element_coefficients->power_sum, __pyx_v_constants->element_coefficients->elementary_symmetric_polynomial, __pyx_v_constants->order);

    /* "brainpy/_c/isotopic_constants.pyx":294
 *                constants.element_coefficients.elementary_symmetric_polynomial,
 *                constants.order)
 *         newton(constants.mass_coefficients.power_sum,             # <<<<<<<<<<<<<<
 *                constants.mass_coefficients.elementary_symmetric_polynomial,
 *                constants.order)
 */
    __pyx_f_7brainpy_2_c_18isotopic_constants_newton(__pyx_v_constants->mass_coefficients->power_sum, __pyx_v_constants->mass_coefficients->elementary_symmetric_polynomial, __pyx_v_constants->order);
    __pyx_L3_continue:;
  }

  /* "brainpy/_c/isotopic_constants.pyx":274
 * 
 * 
 * cdef void isotopic_constants_update_coefficients(IsotopicConstants* isotopes) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j
 */

  /* function exit code */
}

/* "brainpy/_c/isotopic_constants.pyx":299
 * 
 * 
 * cdef double isotopic_constants_nth_element_power_sum(IsotopicConstants* isotopes, char* symbol, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         PhiConstants* constants
 */

static double __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_nth_element_power_sum(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_isotopes, char *__pyx_v_symbol, int __pyx_v_order) {
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *__pyx_v_constants;
  double __pyx_r;

  /* "brainpy/_c/isotopic_constants.pyx":302
 *     cdef:
 *         PhiConstants* constants
 *     isotopic_constants_get(isotopes, symbol, &constants)             # <<<<<<<<<<<<<<
 *     return constants.element_coefficients.power_sum.v[order]
 * 
 */
  (void)(__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get(__pyx_v_isotopes, __pyx_v_symbol, (&__pyx_v_constants)));

  /* "brainpy/_c/isotopic_constants.pyx":303
 *         PhiConstants* constants
 *     isotopic_constants_get(isotopes, symbol, &constants)
 *     return constants.element_coefficients.power_sum.v[order]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (__pyx_v_constants->element_coefficients->power_sum->v[__pyx_v_order]);
  goto __pyx_L0;

  /* "brainpy/_c/isotopic_constants.pyx":299
 * 
 * 
 * cdef double isotopic_constants_nth_element_power_sum(IsotopicConstants* isotopes, char* symbol, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         PhiConstants* constants
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":306
 * 
 * 
 * cdef double isotopic_constants_nth_element_power_sum_by_index(IsotopicConstants* isotopes, size_t index, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         PhiConstants* constants
 */

static double __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_nth_element_power_sum_by_index(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_isotopes, size_t __pyx_v_index, int __pyx_v_order) {
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *__pyx_v_constants;
  double __pyx_r;

  /* "brainpy/_c/isotopic_constants.pyx":309
 *     cdef:
 *         PhiConstants* constants
 *     isotopic_constants_get_by_index(isotopes, index, &constants)             # <<<<<<<<<<<<<<
 *     return constants.element_coefficients.power_sum.v[order]
 * 
 */
  (void)(__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get_by_index(__pyx_v_isotopes, __pyx_v_index, (&__pyx_v_constants)));

  /* "brainpy/_c/isotopic_constants.pyx":310
 *         PhiConstants* constants
 *     isotopic_constants_get_by_index(isotopes, index, &constants)
 *     return constants.element_coefficients.power_sum.v[order]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (__pyx_v_constants->element_coefficients->power_sum->v[__pyx_v_order]);
  goto __pyx_L0;

  /* "brainpy/_c/isotopic_constants.pyx":306
 * 
 * 
 * cdef double isotopic_constants_nth_element_power_sum_by_index(IsotopicConstants* isotopes, size_t index, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         PhiConstants* constants
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":313
 * 
 * 
 * cdef double isotopic_constants_nth_modified_element_power_sum(IsotopicConstants* isotopes, char* symbol, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         PhiConstants* constants
 */

static double __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_nth_modified_element_power_sum(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_isotopes, char *__pyx_v_symbol, int __pyx_v_order) {
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *__pyx_v_constants;
  double __pyx_r;

  /* "brainpy/_c/isotopic_constants.pyx":316
 *     cdef:
 *         PhiConstants* constants
 *     isotopic_constants_get(isotopes, symbol, &constants)             # <<<<<<<<<<<<<<
 *     return constants.mass_coefficients.power_sum.v[order]
 * 
 */
  (void)(__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get(__pyx_v_isotopes, __pyx_v_symbol, (&__pyx_v_constants)));

  /* "brainpy/_c/isotopic_constants.pyx":317
 *         PhiConstants* constants
 *     isotopic_constants_get(isotopes, symbol, &constants)
 *     return constants.mass_coefficients.power_sum.v[order]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (__pyx_v_constants->mass_coefficients->power_sum->v[__pyx_v_order]);
  goto __pyx_L0;

  /* "brainpy/_c/isotopic_constants.pyx":313
 * 
 * 
 * cdef double isotopic_constants_nth_modified_element_power_sum(IsotopicConstants* isotopes, char* symbol, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         PhiConstants* constants
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":320
 * 
 * 
 * cdef double isotopic_constants_nth_modified_element_power_sum_by_index(IsotopicConstants* isotopes, size_t index, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         PhiConstants* constants
 */

static double __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_nth_modified_element_power_sum_by_index(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_isotopes, size_t __pyx_v_index, int __pyx_v_order) {
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *__pyx_v_constants;
  double __pyx_r;

  /* "brainpy/_c/isotopic_constants.pyx":323
 *     cdef:
 *         PhiConstants* constants
 *     isotopic_constants_get_by_index(isotopes, index, &constants)             # <<<<<<<<<<<<<<
 *     return constants.mass_coefficients.power_sum.v[order]
 * 
 */
  (void)(__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get_by_index(__pyx_v_isotopes, __pyx_v_index, (&__pyx_v_constants)));

  /* "brainpy/_c/isotopic_constants.pyx":324
 *         PhiConstants* constants
 *     isotopic_constants_get_by_index(isotopes, index, &constants)
 *     return constants.mass_coefficients.power_sum.v[order]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (__pyx_v_constants->mass_coefficients->power_sum->v[__pyx_v_order]);
  goto __pyx_L0;

  /* "brainpy/_c/isotopic_constants.pyx":320
 * 
 * 
 * cdef double isotopic_constants_nth_modified_element_power_sum_by_index(IsotopicConstants* isotopes, size_t index, int order) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         PhiConstants* constants
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "brainpy/_c/isotopic_constants.pyx":327
 * 
 * 
 * cdef void print_isotopic_constants(IsotopicConstants* isotopes) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

static void __pyx_f_7brainpy_2_c_18isotopic_constants_print_isotopic_constants(struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_isotopes) {
  size_t __pyx_v_i;
  size_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;

  /* "brainpy/_c/isotopic_constants.pyx":331
 *         size_t i
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     for i in range(isotopes.used):
 *         printf("%d\n", i)
 */
  __pyx_v_i = 0;

  /* "brainpy/_c/isotopic_constants.pyx":332
 * 
 *     i = 0
 *     for i in range(isotopes.used):             # <<<<<<<<<<<<<<
 *         printf("%d\n", i)
 *         print_phi_constants(isotopes.constants[i])
 */
  __pyx_t_1 = __pyx_v_isotopes->used;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "brainpy/_c/isotopic_constants.pyx":333
 *     i = 0
 *     for i in range(isotopes.used):
 *         printf("%d\n", i)             # <<<<<<<<<<<<<<
 *         print_phi_constants(isotopes.constants[i])
 * 
 */
    (void)(printf(((char const *)"%d\n"), __pyx_v_i));

    /* "brainpy/_c/isotopic_constants.pyx":334
 *     for i in range(isotopes.used):
 *         printf("%d\n", i)
 *         print_phi_constants(isotopes.constants[i])             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_f_7brainpy_2_c_18isotopic_constants_print_phi_constants((__pyx_v_isotopes->constants[__pyx_v_i]));
  }

  /* "brainpy/_c/isotopic_constants.pyx":327
 * 
 * 
 * cdef void print_isotopic_constants(IsotopicConstants* isotopes) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

  /* function exit code */
}

/* "brainpy/_c/isotopic_constants.pyx":337
 * 
 * 
 * def main():             # <<<<<<<<<<<<<<
 *     cdef:
 *         IsotopicConstants* ic
 */

/* Python wrapper */
static PyObject *__pyx_pw_7brainpy_2_c_18isotopic_constants_1main(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7brainpy_2_c_18isotopic_constants_main[] = "main()";
static PyMethodDef __pyx_mdef_7brainpy_2_c_18isotopic_constants_1main = {"main", (PyCFunction)__pyx_pw_7brainpy_2_c_18isotopic_constants_1main, METH_NOARGS, __pyx_doc_7brainpy_2_c_18isotopic_constants_main};
static PyObject *__pyx_pw_7brainpy_2_c_18isotopic_constants_1main(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("main (wrapper)", 0);
  __pyx_r = __pyx_pf_7brainpy_2_c_18isotopic_constants_main(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7brainpy_2_c_18isotopic_constants_main(CYTHON_UNUSED PyObject *__pyx_self) {
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *__pyx_v_ic;
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *__pyx_v_constant;
  char *__pyx_v_sym;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("main", 0);

  /* "brainpy/_c/isotopic_constants.pyx":346
 *         char* sym
 * 
 *     sym = "O"             # <<<<<<<<<<<<<<
 *     ic = make_isotopic_constants()
 *     print(ic.used)
 */
  __pyx_v_sym = ((char *)"O");

  /* "brainpy/_c/isotopic_constants.pyx":347
 * 
 *     sym = "O"
 *     ic = make_isotopic_constants()             # <<<<<<<<<<<<<<
 *     print(ic.used)
 *     isotopic_constants_add_element(ic, sym)
 */
  __pyx_v_ic = __pyx_f_7brainpy_2_c_18isotopic_constants_make_isotopic_constants();

  /* "brainpy/_c/isotopic_constants.pyx":348
 *     sym = "O"
 *     ic = make_isotopic_constants()
 *     print(ic.used)             # <<<<<<<<<<<<<<
 *     isotopic_constants_add_element(ic, sym)
 *     print(ic.used)
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_ic->used); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PrintOne(0, __pyx_t_1) < 0) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "brainpy/_c/isotopic_constants.pyx":349
 *     ic = make_isotopic_constants()
 *     print(ic.used)
 *     isotopic_constants_add_element(ic, sym)             # <<<<<<<<<<<<<<
 *     print(ic.used)
 *     isotopic_constants_add_element(ic, "C")
 */
  __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_add_element(__pyx_v_ic, __pyx_v_sym);

  /* "brainpy/_c/isotopic_constants.pyx":350
 *     print(ic.used)
 *     isotopic_constants_add_element(ic, sym)
 *     print(ic.used)             # <<<<<<<<<<<<<<
 *     isotopic_constants_add_element(ic, "C")
 *     isotopic_constants_add_element(ic, "H")
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_ic->used); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PrintOne(0, __pyx_t_1) < 0) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "brainpy/_c/isotopic_constants.pyx":351
 *     isotopic_constants_add_element(ic, sym)
 *     print(ic.used)
 *     isotopic_constants_add_element(ic, "C")             # <<<<<<<<<<<<<<
 *     isotopic_constants_add_element(ic, "H")
 * 
 */
  __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_add_element(__pyx_v_ic, ((char *)"C"));

  /* "brainpy/_c/isotopic_constants.pyx":352
 *     print(ic.used)
 *     isotopic_constants_add_element(ic, "C")
 *     isotopic_constants_add_element(ic, "H")             # <<<<<<<<<<<<<<
 * 
 *     if isotopic_constants_get(ic, "O", &constant) == 0:
 */
  __pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_add_element(__pyx_v_ic, ((char *)"H"));

  /* "brainpy/_c/isotopic_constants.pyx":354
 *     isotopic_constants_add_element(ic, "H")
 * 
 *     if isotopic_constants_get(ic, "O", &constant) == 0:             # <<<<<<<<<<<<<<
 *         print_phi_constants(constant)
 *     else:
 */
  __pyx_t_2 = ((__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get(__pyx_v_ic, ((char *)"O"), (&__pyx_v_constant)) == 0) != 0);
  if (__pyx_t_2) {

    /* "brainpy/_c/isotopic_constants.pyx":355
 * 
 *     if isotopic_constants_get(ic, "O", &constant) == 0:
 *         print_phi_constants(constant)             # <<<<<<<<<<<<<<
 *     else:
 *         print("Nope")
 */
    __pyx_f_7brainpy_2_c_18isotopic_constants_print_phi_constants(__pyx_v_constant);

    /* "brainpy/_c/isotopic_constants.pyx":354
 *     isotopic_constants_add_element(ic, "H")
 * 
 *     if isotopic_constants_get(ic, "O", &constant) == 0:             # <<<<<<<<<<<<<<
 *         print_phi_constants(constant)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "brainpy/_c/isotopic_constants.pyx":357
 *         print_phi_constants(constant)
 *     else:
 *         print("Nope")             # <<<<<<<<<<<<<<
 * 
 *     free_isotopic_constants(ic)
 */
  /*else*/ {
    if (__Pyx_PrintOne(0, __pyx_n_s_Nope) < 0) __PYX_ERR(0, 357, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "brainpy/_c/isotopic_constants.pyx":359
 *         print("Nope")
 * 
 *     free_isotopic_constants(ic)             # <<<<<<<<<<<<<<
 */
  __pyx_f_7brainpy_2_c_18isotopic_constants_free_isotopic_constants(__pyx_v_ic);

  /* "brainpy/_c/isotopic_constants.pyx":337
 * 
 * 
 * def main():             # <<<<<<<<<<<<<<
 *     cdef:
 *         IsotopicConstants* ic
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("brainpy._c.isotopic_constants.main", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_isotopic_constants(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_isotopic_constants},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "isotopic_constants",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_DEFAULT_ISOTOPIC_CONSTANTS_SIZE, __pyx_k_DEFAULT_ISOTOPIC_CONSTANTS_SIZE, sizeof(__pyx_k_DEFAULT_ISOTOPIC_CONSTANTS_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_Nope, __pyx_k_Nope, sizeof(__pyx_k_Nope), 0, 0, 1, 1},
  {&__pyx_n_s_brainpy__c_isotopic_constants, __pyx_k_brainpy__c_isotopic_constants, sizeof(__pyx_k_brainpy__c_isotopic_constants), 0, 0, 1, 1},
  {&__pyx_kp_s_brainpy__c_isotopic_constants_py, __pyx_k_brainpy__c_isotopic_constants_py, sizeof(__pyx_k_brainpy__c_isotopic_constants_py), 0, 0, 1, 0},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_coefficients, __pyx_k_coefficients, sizeof(__pyx_k_coefficients), 0, 0, 1, 1},
  {&__pyx_n_s_constant, __pyx_k_constant, sizeof(__pyx_k_constant), 0, 0, 1, 1},
  {&__pyx_n_s_elem, __pyx_k_elem, sizeof(__pyx_k_elem), 0, 0, 1, 1},
  {&__pyx_n_s_elementary_symmetric_polynomial, __pyx_k_elementary_symmetric_polynomial, sizeof(__pyx_k_elementary_symmetric_polynomial), 0, 0, 1, 1},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_n_s_file, __pyx_k_file, sizeof(__pyx_k_file), 0, 0, 1, 1},
  {&__pyx_n_s_ic, __pyx_k_ic, sizeof(__pyx_k_ic), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_main_2, __pyx_k_main_2, sizeof(__pyx_k_main_2), 0, 0, 1, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_capi, __pyx_k_pyx_capi, sizeof(__pyx_k_pyx_capi), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_sym, __pyx_k_sym, sizeof(__pyx_k_sym), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 39, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "brainpy/_c/isotopic_constants.pyx":337
 * 
 * 
 * def main():             # <<<<<<<<<<<<<<
 *     cdef:
 *         IsotopicConstants* ic
 */
  __pyx_tuple_ = PyTuple_Pack(6, __pyx_n_s_ic, __pyx_n_s_coefficients, __pyx_n_s_elementary_symmetric_polynomial, __pyx_n_s_constant, __pyx_n_s_elem, __pyx_n_s_sym); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);
  __pyx_codeobj__2 = (PyObject*)__Pyx_PyCode_New(0, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple_, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_brainpy__c_isotopic_constants_py, __pyx_n_s_main_2, 337, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__2)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_modinit_global_init_code(void); /*proto*/
static int __Pyx_modinit_variable_export_code(void); /*proto*/
static int __Pyx_modinit_function_export_code(void); /*proto*/
static int __Pyx_modinit_type_init_code(void); /*proto*/
static int __Pyx_modinit_type_import_code(void); /*proto*/
static int __Pyx_modinit_variable_import_code(void); /*proto*/
static int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  if (__Pyx_ExportVoidPtr(__pyx_n_s_DEFAULT_ISOTOPIC_CONSTANTS_SIZE, (void *)&__pyx_v_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE, "size_t") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("vietes", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_vietes, "__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *(__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_update_power_sum", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants__update_power_sum, "void (__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, int)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_update_elementary_symmetric_polynomial", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants__update_elementary_symmetric_polynomial, "void (__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, int)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("newton", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_newton, "void (__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *, int)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("compute_isotopic_coefficients", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_compute_isotopic_coefficients, "__pyx_t_7brainpy_2_c_18isotopic_constants_dvec *(struct __pyx_t_7brainpy_2_c_11composition_Element *, int, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("make_polynomial_parameters", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_make_polynomial_parameters, "struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *(struct __pyx_t_7brainpy_2_c_11composition_Element *, int, __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("print_polynomial_parameters", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_print_polynomial_parameters, "void (struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("free_polynomial_parameters", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_free_polynomial_parameters, "void (struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("print_phi_constants", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_print_phi_constants, "void (struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("free_phi_constants", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_free_phi_constants, "void (struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("make_isotopic_constants", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_make_isotopic_constants, "struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *(void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("isotopic_constants_resize", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_resize, "int (struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("free_isotopic_constants", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_free_isotopic_constants, "void (struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("isotopic_constants_add_element", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_add_element, "void (struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *, char *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("isotopic_constants_get", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_get, "int (struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *, char *, struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants **)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("isotopic_constants_update_coefficients", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_update_coefficients, "void (struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("isotopic_constants_nth_element_power_sum", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_nth_element_power_sum, "double (struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *, char *, int)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("isotopic_constants_nth_modified_element_power_sum", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_nth_modified_element_power_sum, "double (struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *, char *, int)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("isotopic_constants_nth_element_power_sum_by_index", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_nth_element_power_sum_by_index, "double (struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *, size_t, int)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("isotopic_constants_nth_modified_element_power_sum_by_index", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_isotopic_constants_nth_modified_element_power_sum_by_index, "double (struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *, size_t, int)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("print_isotopic_constants", (void (*)(void))__pyx_f_7brainpy_2_c_18isotopic_constants_print_isotopic_constants, "void (struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_ptype_7brainpy_2_c_11composition_PyComposition = __Pyx_ImportType("brainpy._c.composition", "PyComposition", sizeof(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition), 1); if (unlikely(!__pyx_ptype_7brainpy_2_c_11composition_PyComposition)) __PYX_ERR(1, 117, __pyx_L1_error)
  __pyx_vtabptr_7brainpy_2_c_11composition_PyComposition = (struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition*)__Pyx_GetVtable(__pyx_ptype_7brainpy_2_c_11composition_PyComposition->tp_dict); if (unlikely(!__pyx_vtabptr_7brainpy_2_c_11composition_PyComposition)) __PYX_ERR(1, 117, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("brainpy._c.composition"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "nist_mass", (void **)&__pyx_vp_7brainpy_2_c_11composition_nist_mass, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "PROTON", (void **)&__pyx_vp_7brainpy_2_c_11composition_PROTON, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_ElementTable", (void **)&__pyx_vp_7brainpy_2_c_11composition__ElementTable, "struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("brainpy._c.composition"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "_parse_isotope_string", (void (**)(void))&__pyx_f_7brainpy_2_c_11composition__parse_isotope_string, "char *(char *, int *, char *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "element_max_neutron_shift", (void (**)(void))&__pyx_f_7brainpy_2_c_11composition_element_max_neutron_shift, "int (struct __pyx_t_7brainpy_2_c_11composition_Element *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "make_fixed_isotope_element", (void (**)(void))&__pyx_f_7brainpy_2_c_11composition_make_fixed_isotope_element, "struct __pyx_t_7brainpy_2_c_11composition_Element *(struct __pyx_t_7brainpy_2_c_11composition_Element *, int)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "make_element_hash_table", (void (**)(void))&__pyx_f_7brainpy_2_c_11composition_make_element_hash_table, "struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *(size_t)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "element_hash_table_get", (void (**)(void))&__pyx_f_7brainpy_2_c_11composition_element_hash_table_get, "int (struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *, char *, struct __pyx_t_7brainpy_2_c_11composition_Element **)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "element_hash_table_put", (void (**)(void))&__pyx_f_7brainpy_2_c_11composition_element_hash_table_put, "int (struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *, struct __pyx_t_7brainpy_2_c_11composition_Element *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "free_element_hash_table", (void (**)(void))&__pyx_f_7brainpy_2_c_11composition_free_element_hash_table, "size_t (struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __Pyx_ImportModule("brainpy._c.double_vector"); if (!__pyx_t_2) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "make_double_vector_with_size", (void (**)(void))&__pyx_f_7brainpy_2_c_13double_vector_make_double_vector_with_size, "struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *(size_t)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "make_double_vector", (void (**)(void))&__pyx_f_7brainpy_2_c_13double_vector_make_double_vector, "struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *(void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "double_vector_append", (void (**)(void))&__pyx_f_7brainpy_2_c_13double_vector_double_vector_append, "int (struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *, double)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "free_double_vector", (void (**)(void))&__pyx_f_7brainpy_2_c_13double_vector_free_double_vector, "void (struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "print_double_vector", (void (**)(void))&__pyx_f_7brainpy_2_c_13double_vector_print_double_vector, "void (struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "reset_double_vector", (void (**)(void))&__pyx_f_7brainpy_2_c_13double_vector_reset_double_vector, "void (struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "double_vector_to_list", (void (**)(void))&__pyx_f_7brainpy_2_c_13double_vector_double_vector_to_list, "PyObject *(struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (!(defined(__cplusplus)) || (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 4)))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initisotopic_constants(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initisotopic_constants(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_isotopic_constants(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_isotopic_constants(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_isotopic_constants(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_isotopic_constants(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("isotopic_constants", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_brainpy___c__isotopic_constants) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "brainpy._c.isotopic_constants")) {
      if (unlikely(PyDict_SetItemString(modules, "brainpy._c.isotopic_constants", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  if (unlikely(__Pyx_modinit_variable_export_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_function_export_code() != 0)) goto __pyx_L1_error;
  (void)__Pyx_modinit_type_init_code();
  if (unlikely(__Pyx_modinit_type_import_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_variable_import_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_function_import_code() != 0)) goto __pyx_L1_error;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "brainpy/_c/isotopic_constants.pyx":182
 * # IsotopicConstants Methods
 * 
 * cdef size_t DEFAULT_ISOTOPIC_CONSTANTS_SIZE = 7             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE = 7;

  /* "brainpy/_c/isotopic_constants.pyx":337
 * 
 * 
 * def main():             # <<<<<<<<<<<<<<
 *     cdef:
 *         IsotopicConstants* ic
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_7brainpy_2_c_18isotopic_constants_1main, NULL, __pyx_n_s_brainpy__c_isotopic_constants); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_main_2, __pyx_t_1) < 0) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "brainpy/_c/isotopic_constants.pyx":1
 * # cython: embedsignature=True             # <<<<<<<<<<<<<<
 * 
 * cimport cython
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init brainpy._c.isotopic_constants", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init brainpy._c.isotopic_constants");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* Print */
#if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static PyObject *__Pyx_GetStdout(void) {
    PyObject *f = PySys_GetObject((char *)"stdout");
    if (!f) {
        PyErr_SetString(PyExc_RuntimeError, "lost sys.stdout");
    }
    return f;
}
static int __Pyx_Print(PyObject* f, PyObject *arg_tuple, int newline) {
    int i;
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    for (i=0; i < PyTuple_GET_SIZE(arg_tuple); i++) {
        PyObject* v;
        if (PyFile_SoftSpace(f, 1)) {
            if (PyFile_WriteString(" ", f) < 0)
                goto error;
        }
        v = PyTuple_GET_ITEM(arg_tuple, i);
        if (PyFile_WriteObject(v, f, Py_PRINT_RAW) < 0)
            goto error;
        if (PyString_Check(v)) {
            char *s = PyString_AsString(v);
            Py_ssize_t len = PyString_Size(v);
            if (len > 0) {
                switch (s[len-1]) {
                    case ' ': break;
                    case '\f': case '\r': case '\n': case '\t': case '\v':
                        PyFile_SoftSpace(f, 0);
                        break;
                    default:  break;
                }
            }
        }
    }
    if (newline) {
        if (PyFile_WriteString("\n", f) < 0)
            goto error;
        PyFile_SoftSpace(f, 0);
    }
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
}
#else
static int __Pyx_Print(PyObject* stream, PyObject *arg_tuple, int newline) {
    PyObject* kwargs = 0;
    PyObject* result = 0;
    PyObject* end_string;
    if (unlikely(!__pyx_print)) {
        __pyx_print = PyObject_GetAttr(__pyx_b, __pyx_n_s_print);
        if (!__pyx_print)
            return -1;
    }
    if (stream) {
        kwargs = PyDict_New();
        if (unlikely(!kwargs))
            return -1;
        if (unlikely(PyDict_SetItem(kwargs, __pyx_n_s_file, stream) < 0))
            goto bad;
        if (!newline) {
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                goto bad;
            if (PyDict_SetItem(kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                goto bad;
            }
            Py_DECREF(end_string);
        }
    } else if (!newline) {
        if (unlikely(!__pyx_print_kwargs)) {
            __pyx_print_kwargs = PyDict_New();
            if (unlikely(!__pyx_print_kwargs))
                return -1;
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                return -1;
            if (PyDict_SetItem(__pyx_print_kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                return -1;
            }
            Py_DECREF(end_string);
        }
        kwargs = __pyx_print_kwargs;
    }
    result = PyObject_Call(__pyx_print, arg_tuple, kwargs);
    if (unlikely(kwargs) && (kwargs != __pyx_print_kwargs))
        Py_DECREF(kwargs);
    if (!result)
        return -1;
    Py_DECREF(result);
    return 0;
bad:
    if (kwargs != __pyx_print_kwargs)
        Py_XDECREF(kwargs);
    return -1;
}
#endif

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* PrintOne */
#if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static int __Pyx_PrintOne(PyObject* f, PyObject *o) {
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    if (PyFile_SoftSpace(f, 0)) {
        if (PyFile_WriteString(" ", f) < 0)
            goto error;
    }
    if (PyFile_WriteObject(o, f, Py_PRINT_RAW) < 0)
        goto error;
    if (PyFile_WriteString("\n", f) < 0)
        goto error;
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
    /* the line below is just to avoid C compiler
     * warnings about unused functions */
    return __Pyx_Print(f, NULL, 0);
}
#else
static int __Pyx_PrintOne(PyObject* stream, PyObject *o) {
    int res;
    PyObject* arg_tuple = PyTuple_Pack(1, o);
    if (unlikely(!arg_tuple))
        return -1;
    res = __Pyx_Print(stream, arg_tuple, 1);
    Py_DECREF(arg_tuple);
    return res;
}
#endif

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* VoidPtrExport */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig) {
    PyObject *d;
    PyObject *cobj = 0;
    d = PyDict_GetItem(__pyx_d, __pyx_n_s_pyx_capi);
    Py_XINCREF(d);
    if (!d) {
        d = PyDict_New();
        if (!d)
            goto bad;
        if (__Pyx_PyObject_SetAttrStr(__pyx_m, __pyx_n_s_pyx_capi, d) < 0)
            goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItem(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* FunctionExport */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* ModuleImport */
#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
